import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../../plugins/api.dart';             // 서버 요청용 (Api.serverRequest)
import '../../dialogs/response_dialog.dart'; // 오류/알림 모달
import '../../views/room/waiting_room_team_page.dart';    // 팀전 대기방 예: import 경로
import '../../views/room/waiting_room_private_page.dart'; // 개인전 대기방 예: import 경로

/// 분리된 방 상세 모달
class RoomDetailDialog extends StatefulWidget {
  final Map<String, dynamic> roomData;

  const RoomDetailDialog({Key? key, required this.roomData}) : super(key: key);

  @override
  State<RoomDetailDialog> createState() => _RoomDetailDialogState();
}

class _RoomDetailDialogState extends State<RoomDetailDialog> {
  late String roomTitle;
  late String roomIntro;
  late String roomStatus;   // '대기중'/'진행중'/'종료'
  late String openYn;       // '공개'/'비공개'
  late bool isPrivate;
  late bool isWait;
  late bool isRunning;
  late bool isFinish;

  /// 서버에 전달할 방 번호 / 방 타입
  late int roomSeq;
  late String roomType;     // "private" 또는 "team" 등

  /// 비밀번호 입력 컨트롤러 (비공개 + 대기중일 때만 표시)
  final TextEditingController _pwController = TextEditingController();

  @override
  void initState() {
    super.initState();
    // roomData에서 필요한 필드 추출
    roomTitle  = widget.roomData['room_title']  ?? '(방제목 없음)';
    roomIntro  = widget.roomData['room_intro']  ?? '';
    roomStatus = widget.roomData['room_status'] ?? '대기중';
    openYn     = widget.roomData['open_yn']     ?? '공개';

    // 서버에 전송할 정보
    roomSeq  = widget.roomData['room_seq']  ?? 0;
    roomType = widget.roomData['room_type'] ?? 'private'; 
    // ※ 서버에 "TEAM"/"team" 식으로 넘길지 확인 필요

    // '비공개'이면 true
    isPrivate = (openYn == '비공개');
    // 상태별 Flag
    isWait    = (roomStatus == '대기중');
    isRunning = (roomStatus == '진행중');
    isFinish  = (roomStatus == '종료');
  }

  @override
  Widget build(BuildContext context) {
    return Dialog(
      backgroundColor: Colors.white,
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(16)),
      child: Padding(
        // 흰 배경 + 넉넉한 간격
        padding: const EdgeInsets.symmetric(horizontal: 24, vertical: 24),
        child: Column(
          mainAxisSize: MainAxisSize.min, // 내용만큼 높이
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            // (A) 상단에 방 제목 (가운데 정렬)
            Center(
              child: Text(
                roomTitle,
                style: const TextStyle(
                  fontSize: 18,
                  fontWeight: FontWeight.bold,
                  color: Colors.black,
                ),
              ),
            ),
            const SizedBox(height: 20),

            // (B) "방 소개" 라벨
            const Text(
              '방 소개',
              style: TextStyle(
                fontWeight: FontWeight.bold,
                color: Colors.black,
                fontSize: 14,
              ),
            ),
            const SizedBox(height: 6),

            // (C) 방 소개 영역
            Container(
              height: 80,
              padding: const EdgeInsets.all(8),
              child: SingleChildScrollView(
                child: Text(
                  roomIntro.isNotEmpty ? roomIntro : '소개글이 없습니다.',
                  style: const TextStyle(color: Colors.black),
                ),
              ),
            ),
            const SizedBox(height: 16),

            // (D) 공개/비공개 표시
            Text(
              isPrivate ? '비공개방' : '공개방',
              style: const TextStyle(
                fontWeight: FontWeight.bold,
                color: Colors.black,
              ),
            ),
            // (D-1) 비밀번호 필드 (비공개 + 대기중)
            if (isPrivate && isWait) ...[
              const SizedBox(height: 16),
              TextField(
                controller: _pwController,
                obscureText: true,
                decoration: const InputDecoration(
                  labelText: '비밀번호',
                  labelStyle: TextStyle(color: Colors.black),
                  border: OutlineInputBorder(),
                ),
              ),
            ],
            const SizedBox(height: 24),

            // (E) 하단 버튼
            _buildBottomButton(),
          ],
        ),
      ),
    );
  }

  /// 하단 버튼 구역
  Widget _buildBottomButton() {
    if (isWait) {
      // (A) 대기중 -> "입장" + "닫기"
      return Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: [
          _buildBlackButton(
            label: '입장',
            onTap: _onEnterRoom, // 실제 입장 로직
          ),
          _buildBlackButton(
            label: '닫기',
            onTap: () => Navigator.pop(context),
          ),
        ],
      );
    } else if (isRunning) {
      // (B) 진행중 -> "확인" (중앙 정렬)
      return Center(
        child: _buildBlackButton(
          label: '확인',
          onTap: () => Navigator.pop(context),
        ),
      );
    } else {
      // (C) 종료 -> "결과보기", "확인" (두 버튼 크기 동일)
      return Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: [
          SizedBox(
            width: 100,
            child: _buildBlackButton(
              label: '결과보기',
              onTap: () {
                // TODO: 결과보기 로직
                Navigator.pop(context);
              },
            ),
          ),
          SizedBox(
            width: 100,
            child: _buildBlackButton(
              label: '확인',
              onTap: () => Navigator.pop(context),
            ),
          ),
        ],
      );
    }
  }

  /// "입장" 버튼 로직
  Future<void> _onEnterRoom() async {
    final pw = _pwController.text.trim();

    // 서버 API 요청 바디
    final requestBody = {
      "room_seq": "$roomSeq",
      "room_type": roomType, 
      "room_pw": pw, // 비공개 방이면 비번 or 공백
    };

    try {
      final response = await Api.serverRequest(
        uri: '/room/score/enter/room',
        body: requestBody,
      );

      if (response == null || response['result'] != 'OK') {
        // 통신 자체 실패
        showResponseDialog(context, '오류', '방 입장 실패. 서버 통신 오류.');
        return;
      }

      final resp = response['response'] ?? {};
      if (resp['result'] == 'OK') {
        // 성공 -> 모달 닫고 대기방 페이지 이동
        Navigator.pop(context); // 현재 모달 닫기

        // room_type에 따라 개인전/팀전 대기방 이동
        if (roomType.toLowerCase() == 'team') {
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(
              builder: (_) => WaitingRoomTeamPage(
                roomSeq: roomSeq,
                roomType: 'team',
              ),
            ),
          );
        } else {
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(
              builder: (_) => WaitingRoomPrivatePage(
                roomSeq: roomSeq,
                roomType: 'private',
              ),
            ),
          );
        }
      } else {
        // 내부 실패
        final msgTitle = resp['response_info']?['msg_title'] ?? '방 입장 실패';
        final msgContent = resp['response_info']?['msg_content'] ?? '오류가 발생했습니다.';
        showResponseDialog(context, msgTitle, msgContent);
      }
    } catch (e) {
      showResponseDialog(context, '오류', '서버 요청 중 예외 발생: $e');
    }
  }

  /// 공통 버튼 스타일
  Widget _buildBlackButton({
    required String label,
    required VoidCallback onTap,
  }) {
    return ElevatedButton(
      onPressed: onTap,
      style: ElevatedButton.styleFrom(
        backgroundColor: Colors.black,
        foregroundColor: Colors.white,
        padding: const EdgeInsets.symmetric(horizontal: 24, vertical: 12),
        shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
      ),
      child: Text(label, style: const TextStyle(fontSize: 14)),
    );
  }
}
