import 'package:flutter/material.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../../plugins/api.dart'; // 서버 API 요청
import 'response_dialog.dart';   // 요청 결과 모달창

class RoomSettingModal extends StatefulWidget {
  final Map<String, dynamic> roomInfo; 
  // 예: {
  //   "room_seq": "13",
  //   "room_master_yn": "Y",
  //   "room_title": "...",
  //   "room_type": "private" or "team"
  //   ...
  // }

  const RoomSettingModal({Key? key, required this.roomInfo}) : super(key: key);

  @override
  State<RoomSettingModal> createState() => _RoomSettingModalState();
}

class _RoomSettingModalState extends State<RoomSettingModal> {
  // ─────────────────────────────────────────────
  // 로컬 상태
  // ─────────────────────────────────────────────
  late bool isMaster;        // 방장 여부
  String openYn = 'Y';       // 공개/비공개
  String roomPw = '';        // 비공개 시 비번
  late int roomSeq;          // 방 번호
  String roomTitle = '';     // 방 제목
  String roomIntro = '';     // 방 소개
  int runningTime = 1;       // 운영 시간
  int numberOfPeople = 10;   // 최대 인원
  String scoreOpenRange = 'PRIVATE'; // 점수 공개 범위 (PRIVATE / TEAM / ALL)

  // FRD 관련
  late DatabaseReference _roomRef;
  bool _isLoading = true;

  // 방이 개인전인지 팀전인지 구분(편의를 위해 로컬 변수 사용)
  late bool isPrivateType; // true이면 개인전, false이면 팀전

  @override
  void initState() {
    super.initState();

    // (1) room_seq
    roomSeq = int.tryParse('${widget.roomInfo['room_seq'] ?? '0'}') ?? 0;

    // (2) 방 타입
    final roomTypeStr = (widget.roomInfo['room_type'] ?? 'private').toString().toLowerCase();
    // room_type 이 "private"면 개인전, 아니면 팀전으로 처리 (예: "team")
    isPrivateType = (roomTypeStr == 'private');

    // (3) firebase ref
    final roomKey = 'korea-$roomSeq';
    _roomRef = FirebaseDatabase.instance.ref('rooms/$roomKey/roomInfo');

    // (4) 내 user_seq와 방장 user_seq 비교 + FRD에서 roomInfo 조회
    _checkMasterAndFetchData();
  }

  /// 로컬스토리지에서 my_user_seq를 불러오고,
  /// FRD에서 roomInfo를 읽어서 state 업데이트
  Future<void> _checkMasterAndFetchData() async {
    final prefs = await SharedPreferences.getInstance();
    final myUserSeq = prefs.getInt('my_user_seq') ?? 0;

    final snapshot = await _roomRef.get(); 
    if (!snapshot.exists) {
      // 방 정보 없음
      setState(() {
        _isLoading = false;
        isMaster = false;
        roomTitle = '방 정보 없음';
      });
      return;
    }

    final data = snapshot.value as Map<dynamic, dynamic>? ?? {};
    // master_user_seq, open_yn, etc
    final masterSeq = data['master_user_seq'] ?? 0;

    setState(() {
      isMaster = (masterSeq.toString() == myUserSeq.toString());

      // 각 필드들
      roomTitle       = data['room_title']?.toString()       ?? '';
      roomIntro       = data['room_intro']?.toString()       ?? '';
      openYn          = data['open_yn']?.toString()          ?? 'Y';
      roomPw          = data['room_pw']?.toString()          ?? '';
      runningTime     = _toInt(data['running_time'], 1);
      numberOfPeople  = _toInt(data['number_of_people'], 10);
      scoreOpenRange  = data['score_open_range']?.toString() ?? 'PRIVATE';

      _isLoading = false;
    });
  }

  /// 단순 int 변환 유틸
  int _toInt(dynamic val, int defaultVal) {
    if (val == null) return defaultVal;
    if (val is int) return val;
    if (val is String) {
      return int.tryParse(val) ?? defaultVal;
    }
    return defaultVal;
  }

  /// (수정) 버튼 클릭
  Future<void> _onUpdate() async {
    // 서버 API로 방 설정 수정
    final requestBody = {
      'room_seq': '$roomSeq',
      'room_status': 'WAIT', 
      'room_title': roomTitle,
      'room_intro': roomIntro,
      'open_yn': openYn,
      'room_pw': roomPw,
      'running_time': '$runningTime',
      // widget.roomInfo['room_type']가 있다면 그대로 전송
      'room_type': widget.roomInfo['room_type'] ?? 'private',
      'number_of_people': '$numberOfPeople',
      'score_open_range': scoreOpenRange,
    };

    // (팀전인 경우에만 number_of_teams 추가)
    if (!isPrivateType) {
      // 예: 임의로 4 라고 가정
      // 실제로는 FRD에서 roomInfo['number_of_teams'] 를 가져올 수도 있음
      requestBody['number_of_teams'] = '4';
    }

    try {
      final response = await Api.serverRequest(
        uri: '/room/score/update/room/setting/info',
        body: requestBody,
      );

      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        final serverResult = resp['result'] ?? 'FAIL';
        if (serverResult == 'OK') {
          await showResponseDialog(
            context,
            '성공',
            '방 설정이 성공적으로 수정되었습니다.',
          );
          Navigator.pop(context, 'refresh');
        } else {
          // 내부 실패
          final msgTitle = resp['response_info']?['msg_title'] ?? '수정 실패';
          final msgContent = resp['response_info']?['msg_content'] ?? '오류가 발생했습니다.';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신에 실패했습니다.');
      }
    } catch (e) {
      showResponseDialog(context, '오류 발생', '서버 요청 중 오류가 발생했습니다.\n$e');
    }
  }

  @override
  Widget build(BuildContext context) {
    return Dialog(
      backgroundColor: Colors.white,
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(16)),
      child: _isLoading
          ? const SizedBox(
              height: 100,
              child: Center(child: CircularProgressIndicator()),
            )
          : SingleChildScrollView(
              padding: const EdgeInsets.symmetric(horizontal: 16, vertical: 12),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  // 모달 상단 제목
                  Text(
                    '방 설정 정보',
                    style: TextStyle(
                      fontSize: 18,
                      fontWeight: FontWeight.bold,
                      color: Colors.black.withOpacity(0.9),
                    ),
                  ),
                  const SizedBox(height: 12),

                  // (1) 방 제목
                  _buildTitle('방 제목'),
                  TextField(
                    readOnly: !isMaster,
                    controller: TextEditingController(text: roomTitle),
                    onChanged: (value) => roomTitle = value,
                    style: const TextStyle(color: Colors.black),
                    decoration: InputDecoration(
                      border: OutlineInputBorder(
                        borderSide: BorderSide(color: Colors.black.withOpacity(0.8)),
                      ),
                      contentPadding: const EdgeInsets.symmetric(horizontal: 12, vertical: 8),
                      isDense: true,
                    ),
                  ),
                  const SizedBox(height: 12),

                  // (2) 방 소개
                  _buildTitle('방 소개'),
                  SizedBox(
                    height: 60, 
                    child: TextField(
                      readOnly: !isMaster,
                      controller: TextEditingController(text: roomIntro),
                      onChanged: (value) => roomIntro = value,
                      maxLines: null,
                      expands: true,
                      style: const TextStyle(color: Colors.black),
                      textAlignVertical: TextAlignVertical.top,
                      decoration: InputDecoration(
                        border: OutlineInputBorder(
                          borderSide: BorderSide(color: Colors.black.withOpacity(0.8)),
                        ),
                        contentPadding: const EdgeInsets.symmetric(horizontal: 12, vertical: 8),
                        isDense: true,
                      ),
                    ),
                  ),
                  const SizedBox(height: 12),

                  // (3) 비밀번호 설정 (공개/비공개)
                  _buildTitle('비밀번호 설정'),
                  Row(
                    children: [
                      Radio<String>(
                        value: 'Y',
                        groupValue: openYn,
                        activeColor: Colors.black,
                        onChanged: isMaster
                            ? (val) {
                                setState(() {
                                  openYn = val ?? 'Y';
                                });
                              }
                            : null,
                      ),
                      const Text('공개', style: TextStyle(color: Colors.black)),
                      const SizedBox(width: 8),
                      Radio<String>(
                        value: 'N',
                        groupValue: openYn,
                        activeColor: Colors.black,
                        onChanged: isMaster
                            ? (val) {
                                setState(() {
                                  openYn = val ?? 'N';
                                });
                              }
                            : null,
                      ),
                      const Text('비공개', style: TextStyle(color: Colors.black)),
                    ],
                  ),
                  if (openYn == 'N') ...[
                    SizedBox(
                      height: 40,
                      child: TextField(
                        readOnly: !isMaster,
                        obscureText: true,
                        onChanged: (value) => roomPw = value,
                        style: const TextStyle(color: Colors.black),
                        decoration: InputDecoration(
                          border: OutlineInputBorder(
                            borderSide: BorderSide(color: Colors.black.withOpacity(0.8)),
                          ),
                          hintText: '비밀번호 입력',
                          hintStyle: TextStyle(color: Colors.black.withOpacity(0.4)),
                          contentPadding: const EdgeInsets.symmetric(horizontal: 12, vertical: 8),
                          isDense: true,
                        ),
                      ),
                    ),
                    const SizedBox(height: 12),
                  ],

                  // (4) 운영시간
                  _buildTitle('운영시간'),
                  Row(
                    children: [
                      DropdownButton<int>(
                        value: runningTime,
                        dropdownColor: Colors.white,
                        style: const TextStyle(color: Colors.black),
                        underline: Container(
                          height: 1,
                          color: Colors.black.withOpacity(0.8),
                        ),
                        items: [1, 2, 3, 4, 5, 6]
                            .map((e) => DropdownMenuItem<int>(
                                  value: e,
                                  child: Text('$e', style: const TextStyle(color: Colors.black)),
                                ))
                            .toList(),
                        onChanged: isMaster
                            ? (val) {
                                if (val == null) return;
                                setState(() {
                                  runningTime = val;
                                });
                              }
                            : null,
                      ),
                      const SizedBox(width: 8),
                      const Text('시간', style: TextStyle(color: Colors.black)),
                    ],
                  ),
                  const SizedBox(height: 12),

                  // (5) 최대 인원수
                  _buildTitle('최대 인원수'),
                  Row(
                    children: [
                      SizedBox(
                        width: 80,
                        height: 40,
                        child: TextField(
                          readOnly: !isMaster,
                          controller: TextEditingController(text: '$numberOfPeople'),
                          keyboardType: TextInputType.number,
                          onChanged: (value) {
                            setState(() {
                              numberOfPeople = int.tryParse(value) ?? numberOfPeople;
                            });
                          },
                          style: const TextStyle(color: Colors.black),
                          decoration: InputDecoration(
                            border: OutlineInputBorder(
                              borderSide: BorderSide(color: Colors.black.withOpacity(0.8)),
                            ),
                            contentPadding: const EdgeInsets.symmetric(horizontal: 12, vertical: 8),
                            isDense: true,
                          ),
                        ),
                      ),
                      const SizedBox(width: 8),
                      const Text('명', style: TextStyle(color: Colors.black)),
                    ],
                  ),
                  const SizedBox(height: 12),

                  // (6) 점수 공개 범위
                  _buildTitle('점수 공개 범위'),
                  // 개인전이면 PRIVATE, ALL 두 가지만 노출
                  // 팀전이면 PRIVATE, TEAM, ALL 세 가지 노출
                  if (isPrivateType)
                    Column(
                      children: [
                        // PRIVATE
                        RadioListTile<String>(
                          value: 'PRIVATE',
                          groupValue: scoreOpenRange,
                          activeColor: Colors.black,
                          title: const Text('개인', style: TextStyle(color: Colors.black)),
                          onChanged: isMaster
                              ? (val) {
                                  if (val != null) {
                                    setState(() => scoreOpenRange = val);
                                  }
                                }
                              : null,
                        ),
                        // ALL
                        RadioListTile<String>(
                          value: 'ALL',
                          groupValue: scoreOpenRange,
                          activeColor: Colors.black,
                          title: const Text('전체', style: TextStyle(color: Colors.black)),
                          onChanged: isMaster
                              ? (val) {
                                  if (val != null) {
                                    setState(() => scoreOpenRange = val);
                                  }
                                }
                              : null,
                        ),
                      ],
                    )
                  else
                    // 팀전이면 PRIVATE / TEAM / ALL
                    Column(
                      children: [
                        RadioListTile<String>(
                          value: 'PRIVATE',
                          groupValue: scoreOpenRange,
                          activeColor: Colors.black,
                          title: const Text('개인', style: TextStyle(color: Colors.black)),
                          onChanged: isMaster
                              ? (val) {
                                  if (val != null) {
                                    setState(() => scoreOpenRange = val);
                                  }
                                }
                              : null,
                        ),
                        RadioListTile<String>(
                          value: 'TEAM',
                          groupValue: scoreOpenRange,
                          activeColor: Colors.black,
                          title: const Text('팀', style: TextStyle(color: Colors.black)),
                          onChanged: isMaster
                              ? (val) {
                                  if (val != null) {
                                    setState(() => scoreOpenRange = val);
                                  }
                                }
                              : null,
                        ),
                        RadioListTile<String>(
                          value: 'ALL',
                          groupValue: scoreOpenRange,
                          activeColor: Colors.black,
                          title: const Text('전체', style: TextStyle(color: Colors.black)),
                          onChanged: isMaster
                              ? (val) {
                                  if (val != null) {
                                    setState(() => scoreOpenRange = val);
                                  }
                                }
                              : null,
                        ),
                      ],
                    ),
                  const SizedBox(height: 20),

                  // (7) 하단 버튼
                  if (isMaster)
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: [
                        ElevatedButton(
                          onPressed: _onUpdate,
                          style: ElevatedButton.styleFrom(
                            backgroundColor: Colors.black,
                            padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 12),
                          ),
                          child: const Text('수정', style: TextStyle(color: Colors.white)),
                        ),
                        ElevatedButton(
                          onPressed: () => Navigator.pop(context),
                          style: ElevatedButton.styleFrom(
                            backgroundColor: Colors.black,
                            padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 12),
                          ),
                          child: const Text('확인', style: TextStyle(color: Colors.white)),
                        ),
                      ],
                    )
                  else
                    Center(
                      child: ElevatedButton(
                        onPressed: () => Navigator.pop(context),
                        style: ElevatedButton.styleFrom(
                          backgroundColor: Colors.black,
                          padding: const EdgeInsets.symmetric(horizontal: 40, vertical: 12),
                        ),
                        child: const Text('확인', style: TextStyle(color: Colors.white)),
                      ),
                    ),
                ],
              ),
            ),
    );
  }

  /// 블랙 앤 화이트 컨셉의 타이틀(레이블) 텍스트
  Widget _buildTitle(String label) {
    return Align(
      alignment: Alignment.centerLeft,
      child: Text(
        label,
        style: TextStyle(
          fontWeight: FontWeight.bold,
          fontSize: 14,
          color: Colors.black.withOpacity(0.9),
        ),
      ),
    );
  }
}
