import 'package:flutter/material.dart';
import '../plugins/api.dart';       // 서버 요청
import 'response_dialog.dart';     // 응답 모달 etc

class ScoreEditDialog extends StatefulWidget {
  final int roomSeq;
  final String roomType;   // "PRIVATE" or "TEAM"
  final Map<String, dynamic> userData;

  const ScoreEditDialog({
    Key? key,
    required this.roomSeq,
    required this.roomType,
    required this.userData,
  }) : super(key: key);

  @override
  State<ScoreEditDialog> createState() => _ScoreEditDialogState();
}

class _ScoreEditDialogState extends State<ScoreEditDialog> {
  late int currentScore; // 현재 점수
  late int newScore;     // 수정 후 점수

  @override
  void initState() {
    super.initState();
    currentScore = (widget.userData['score'] ?? 0) as int;
    newScore = currentScore;
  }

  Future<void> _onApplyScore() async {
    final reqBody = {
      "room_seq": "${widget.roomSeq}",
      "room_type": widget.roomType,
      "target_user_seq": "${widget.userData['user_seq']}",
      "after_score": "$newScore",
    };

    try {
      final response = await Api.serverRequest(
        uri: '/room/score/update/score',
        body: reqBody,
      );
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          // 성공
          await showResponseDialog(context, '성공', '점수가 업데이트되었습니다.');
          Navigator.pop(context, 'refresh');
        } else {
          final msgTitle = resp['response_info']?['msg_title'] ?? '오류';
          final msgContent = resp['response_info']?['msg_content'] ?? '점수 업데이트 실패';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신 실패');
      }
    } catch (e) {
      showResponseDialog(context, '오류 발생', '$e');
    }
  }

  void _onDelta(int delta) {
    setState(() {
      newScore += delta;
      if (newScore < 0) newScore = 0; // 최소 0점이라고 가정
      if (newScore > 999999) newScore = 999999; // 임의 최대치
    });
  }

  @override
  Widget build(BuildContext context) {
    final userName = widget.userData['nickname'] ?? '유저';
    final department = widget.userData['department'] ?? '';
    final introduce = widget.userData['introduce_myself'] ?? '';

    return Dialog(
      backgroundColor: Colors.white,
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(12)),
      child: Padding(
        padding: const EdgeInsets.all(16),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Text('유저 정보 보기', style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            const SizedBox(height: 12),
            // 닉네임 & 소속
            Text(userName, style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold)),
            Text(department, style: TextStyle(fontSize: 14, color: Colors.grey)),

            const SizedBox(height: 12),
            // 소개
            const Align(
              alignment: Alignment.centerLeft,
              child: Text('소개글', style: TextStyle(fontWeight: FontWeight.bold)),
            ),
            const SizedBox(height: 4),
            Container(
              width: double.infinity,
              height: 80,
              decoration: BoxDecoration(
                border: Border.all(color: Colors.black),
                borderRadius: BorderRadius.circular(8),
              ),
              child: SingleChildScrollView(
                padding: const EdgeInsets.all(8),
                child: Text(introduce.isNotEmpty ? introduce : '소개글이 없습니다.'),
              ),
            ),

            const SizedBox(height: 12),
            // 점수 수정 영역
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Text('$currentScore',
                    style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold)),
                const Text(' → '),
                Text('$newScore',
                    style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold)),
              ],
            ),
            const SizedBox(height: 8),
            Wrap(
              spacing: 4,
              runSpacing: 4,
              children: [
                _buildDeltaButton(-100),
                _buildDeltaButton(-10),
                _buildDeltaButton(-1),
                _buildDeltaButton(1),
                _buildDeltaButton(10),
                _buildDeltaButton(100),
              ],
            ),

            const SizedBox(height: 12),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                ElevatedButton(
                  onPressed: _onApplyScore,
                  style: ElevatedButton.styleFrom(backgroundColor: Colors.black),
                  child: const Text('적용', style: TextStyle(color: Colors.white)),
                ),
                ElevatedButton(
                  onPressed: () => Navigator.pop(context),
                  style: ElevatedButton.styleFrom(backgroundColor: Colors.black),
                  child: const Text('닫기', style: TextStyle(color: Colors.white)),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }

  Widget _buildDeltaButton(int delta) {
    final label = (delta >= 0) ? '+$delta' : '$delta';
    return ElevatedButton(
      onPressed: () => _onDelta(delta),
      style: ElevatedButton.styleFrom(
        backgroundColor: Colors.white,
        foregroundColor: Colors.black,
        side: BorderSide(color: Colors.black),
      ),
      child: Text(label),
    );
  }
}
