import 'package:flutter/material.dart';
import 'response_dialog.dart';
import '../../plugins/api.dart';

class TeamNameEditModal extends StatefulWidget {
  final int roomSeq;                 // 방 번호
  final String roomTypeName;         // "TEAM"
  final String beforeTeamName;       // 현재 팀명
  final List<String> existingTeamNames; // 이미 존재하는 팀명들 (ex: ["A","B","C","WAIT"...]

  const TeamNameEditModal({
    Key? key,
    required this.roomSeq,
    required this.roomTypeName,
    required this.beforeTeamName,
    required this.existingTeamNames,
  }) : super(key: key);

  @override
  State<TeamNameEditModal> createState() => _TeamNameEditModalState();
}

class _TeamNameEditModalState extends State<TeamNameEditModal> {
  String afterTeamName = '';
  String _errorMsg = ''; // 팀명 중복 에러 등 표시

  Future<void> _onUpdateTeamName() async {
    // 새 팀명이 비었거나 기존 팀명과 같으면 처리
    final newName = afterTeamName.trim().toUpperCase();
    if (newName.isEmpty) {
      setState(() {
        _errorMsg = '새 팀명을 입력해주세요.';
      });
      return;
    }
    // 중복 검사 (단, 기존 teamName과 동일하면 OK)
    // 예: beforeTeamName= "A", user가 "B" → existingTeamNames=["A","B","C"] 이면 중복
    final existingNames = widget.existingTeamNames.map((e) => e.toUpperCase()).toList();
    if (newName != widget.beforeTeamName.toUpperCase() && existingNames.contains(newName)) {
      setState(() {
        _errorMsg = '이미 존재하는 팀명입니다.';
      });
      return;
    }

    // 서버 요청 body
    // {
    //   "room_seq": "9",
    //   "room_type_name": "TEAM",
    //   "before_team_name": "A",
    //   "after_team_name": "B"
    // }
    final reqBody = {
      'room_seq': '${widget.roomSeq}',
      'room_type_name': widget.roomTypeName, // "TEAM"
      'before_team_name': widget.beforeTeamName,
      'after_team_name': newName,
    };

    try {
      final response = await Api.serverRequest(
        uri: '/room/score/update/team/name',
        body: reqBody,
      );
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          // 성공
          await showResponseDialog(context, '성공', '팀명이 성공적으로 수정되었습니다.');
          Navigator.pop(context, 'refresh');
        } else {
          final msgTitle = resp['response_info']?['msg_title'] ?? '오류';
          final msgContent = resp['response_info']?['msg_content'] ?? '수정 실패';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신 실패');
      }
    } catch (e) {
      showResponseDialog(context, '오류 발생', '$e');
    }
  }

  @override
  Widget build(BuildContext context) {
    return Dialog(
      backgroundColor: Colors.white,
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(16)),
      child: Padding(
        padding: const EdgeInsets.all(16),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            const Text('팀명 수정',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            const SizedBox(height: 12),
            Text('기존 팀명: ${widget.beforeTeamName}'),
            const SizedBox(height: 8),
            TextField(
              decoration: const InputDecoration(
                border: OutlineInputBorder(),
                labelText: '새 팀명',
              ),
              onChanged: (val) {
                setState(() {
                  afterTeamName = val; 
                  _errorMsg = '';      // 에러 메시지 초기화
                });
              },
            ),
            // 에러 메시지
            if (_errorMsg.isNotEmpty) ...[
              const SizedBox(height: 8),
              Text(
                _errorMsg,
                style: const TextStyle(color: Colors.red),
              ),
            ],
            const SizedBox(height: 20),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                SizedBox(
                  width: 100,
                  child: ElevatedButton(
                    onPressed: _onUpdateTeamName,
                    style: ElevatedButton.styleFrom(backgroundColor: Colors.black),
                    child:
                        const Text('수정', style: TextStyle(color: Colors.white)),
                  ),
                ),
                SizedBox(
                  width: 100,
                  child: ElevatedButton(
                    onPressed: () => Navigator.pop(context),
                    style: ElevatedButton.styleFrom(backgroundColor: Colors.black),
                    child:
                        const Text('취소', style: TextStyle(color: Colors.white)),
                  ),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }
}
