import 'package:flutter/material.dart';
import 'response_dialog.dart';
import '../../plugins/api.dart';

class UserInfoTeamDialog extends StatefulWidget {
  final Map<String, dynamic> userData;  
  final bool isRoomMaster;       // 이 모달을 연 "현재 로그인 유저"가 방장인지
  final int roomSeq;
  final String roomTypeName;     // "TEAM"
  final List<String> teamNameList;

  const UserInfoTeamDialog({
    Key? key,
    required this.userData,
    required this.isRoomMaster,
    required this.roomSeq,
    required this.roomTypeName,
    required this.teamNameList,
  }) : super(key: key);

  @override
  State<UserInfoTeamDialog> createState() => _UserInfoTeamDialogState();
}

class _UserInfoTeamDialogState extends State<UserInfoTeamDialog> {
  late String participantType; // 'ADMIN' / 'PLAYER'
  late String teamName;        // 'A'/'B'/'WAIT'
  late String introduceMyself; // 유저 소개

  @override
  void initState() {
    super.initState();
    final rawType = (widget.userData['participant_type'] ?? 'PLAYER').toString().toUpperCase();
    participantType = (rawType == 'ADMIN') ? 'ADMIN' : 'PLAYER';

    final rawTeam = (widget.userData['team_name'] ?? '').toString().trim().toUpperCase();
    teamName = (rawTeam.isEmpty || rawTeam == 'WAIT') ? 'WAIT' : rawTeam;

    introduceMyself = widget.userData['introduce_myself'] ?? '';

    // teamNameList에 WAIT 없으면 추가 (기존 코드)
    final hasWait = widget.teamNameList.map((e) => e.toUpperCase()).contains('WAIT');
    if (!hasWait) {
      widget.teamNameList.add('WAIT');
    }
  }

  // (1) 역할/팀 수정 API (기존 코드)
  Future<void> _onUpdateUserInfo() async {
    // 방장이 아닌데 수정 시도 -> 그냥 return
    if (!widget.isRoomMaster) return;

    final reqBody = {
      'room_seq': '${widget.roomSeq}',
      'room_type_name': widget.roomTypeName, // "TEAM"
      'target_user_seq': '${widget.userData['user_seq']}',
      'participant_type': participantType,   
      'team_name': teamName,
    };

    try {
      final response = await Api.serverRequest(
        uri: '/room/score/update/user/role',
        body: reqBody,
      );
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          await showResponseDialog(context, '성공', '역할/팀이 성공적으로 수정되었습니다.');
          Navigator.pop(context, 'refresh');
        } else {
          final msgTitle = resp['response_info']?['msg_title'] ?? '오류';
          final msgContent = resp['response_info']?['msg_content'] ?? '수정 실패';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신 실패');
      }
    } catch (e) {
      showResponseDialog(context, '오류 발생', '$e');
    }
  }

  // (2) 새로 추가: "추방하기" API 호출 함수
  Future<void> _onKickParticipant() async {
    // 방장이 아니면 리턴
    if (!widget.isRoomMaster) return;

    final reqBody = {
      "room_seq": "${widget.roomSeq}",
      "target_user_seq": "${widget.userData['user_seq']}",
    };

    try {
      final response = await Api.serverRequest(
        uri: '/room/score/kick/participant',
        body: reqBody,
      );
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          // 성공
          await showResponseDialog(context, '성공', '해당 유저가 강퇴되었습니다.');
          Navigator.pop(context, 'refresh'); 
        } else {
          final msgTitle = resp['response_info']?['msg_title'] ?? '오류';
          final msgContent = resp['response_info']?['msg_content'] ?? '강퇴 실패';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신 실패');
      }
    } catch (e) {
      showResponseDialog(context, '오류 발생', '$e');
    }
  }

  @override
  Widget build(BuildContext context) {
    final userName   = widget.userData['nickname'] ?? '유저';
    final profileImg = widget.userData['profile_img'] ?? '';
    final department = widget.userData['department'] ?? '소속정보 없음';

    return Dialog(
      backgroundColor: Colors.white,
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(16)),
      child: Padding(
        padding: const EdgeInsets.all(16),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            const Text('유저 정보 (팀전)', style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            const SizedBox(height: 12),

            // (A) 프로필 영역
            Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                // 프로필 이미지
                Container(
                  width: 80, height: 80,
                  decoration: BoxDecoration(
                    border: Border.all(color: Colors.black),
                    borderRadius: BorderRadius.circular(16),
                  ),
                  child: ClipRRect(
                    borderRadius: BorderRadius.circular(16),
                    child: (profileImg.isNotEmpty)
                        ? Image.network(
                            'https://eldsoft.com:8097/images$profileImg',
                            fit: BoxFit.cover,
                            errorBuilder: (ctx, err, st) => 
                              const Center(child: Text('이미지\n불가', textAlign: TextAlign.center)),
                          )
                        : const Center(child: Text('이미지\n없음', textAlign: TextAlign.center)),
                  ),
                ),
                const SizedBox(width: 16),
                // 닉네임 + 소속
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(userName, style: const TextStyle(fontSize: 16, fontWeight: FontWeight.bold)),
                      const SizedBox(height: 6),
                      Text(department, style: const TextStyle(fontSize: 14)),
                    ],
                  ),
                ),
              ],
            ),
            const SizedBox(height: 16),

            // (B) 방장만 역할/팀 수정 가능
            if (widget.isRoomMaster) ...[
              // 역할 드롭다운
              Row(
                children: [
                  const Text('역할: ', style: TextStyle(fontWeight: FontWeight.bold)),
                  DropdownButton<String>(
                    value: participantType,
                    items: const [
                      DropdownMenuItem(value: 'ADMIN', child: Text('사회자')),
                      DropdownMenuItem(value: 'PLAYER', child: Text('참가자')),
                    ],
                    onChanged: (val) {
                      if (val == null) return;
                      setState(() {
                        participantType = val;
                      });
                    },
                  ),
                ],
              ),
              const SizedBox(height: 12),

              // 팀명 드롭다운
              Row(
                children: [
                  const Text('팀명: ', style: TextStyle(fontWeight: FontWeight.bold)),
                  DropdownButton<String>(
                    value: widget.teamNameList
                      .map((e) => e.toUpperCase())
                      .contains(teamName) ? teamName : 'WAIT',
                    items: widget.teamNameList.map((t) => DropdownMenuItem(
                      value: t.toUpperCase(),
                      child: Text(t),
                    )).toList(),
                    onChanged: (val) {
                      if (val == null) return;
                      setState(() {
                        teamName = val;
                      });
                    },
                  ),
                ],
              ),
              const SizedBox(height: 16),
            ] else ...[
              // (B') 일반유저 -> 그냥 정보만 표시
              Text('역할: $participantType', style: const TextStyle(fontWeight: FontWeight.bold)),
              const SizedBox(height: 6),
              Text('팀명: $teamName', style: const TextStyle(fontWeight: FontWeight.bold)),
              const SizedBox(height: 16),
            ],

            // (C) 소개
            const Align(
              alignment: Alignment.centerLeft,
              child: Text('소개', style: TextStyle(fontWeight: FontWeight.bold)),
            ),
            const SizedBox(height: 4),
            Container(
              width: double.infinity,
              height: 100,
              decoration: BoxDecoration(
                border: Border.all(color: Colors.black),
                borderRadius: BorderRadius.circular(8),
              ),
              child: SingleChildScrollView(
                padding: const EdgeInsets.all(8),
                child: Text(
                  introduceMyself.isNotEmpty ? introduceMyself : '소개글이 없습니다.',
                  style: const TextStyle(fontSize: 14),
                  softWrap: true,               // 줄바꿈 허용
                  maxLines: 100,               // 임의의 큰 수
                  overflow: TextOverflow.clip,  // 필요하면 clip 처리
                ),
              ),
            ),
            const SizedBox(height: 16),

            // (D) 하단 버튼
            if (widget.isRoomMaster) ...[
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: [
                  // 수정하기
                  SizedBox(
                    width: 90,
                    child: ElevatedButton(
                      onPressed: _onUpdateUserInfo,
                      style: ElevatedButton.styleFrom(backgroundColor: Colors.black),
                      // (★) FittedBox 감싸기
                      child: FittedBox(
                        child: Text(
                          '수정하기',
                          style: TextStyle(color: Colors.white),
                        ),
                      ),
                    ),
                  ),
                  // 추방하기
                  SizedBox(
                    width: 90,
                    child: ElevatedButton(
                      onPressed: _onKickParticipant,
                      style: ElevatedButton.styleFrom(backgroundColor: Colors.black),
                      child: FittedBox(
                        child: Text(
                          '추방하기',
                          style: TextStyle(color: Colors.white),
                        ),
                      ),
                    ),
                  ),
                  // 확인
                  SizedBox(
                    width: 90,
                    child: ElevatedButton(
                      onPressed: () => Navigator.pop(context),
                      style: ElevatedButton.styleFrom(backgroundColor: Colors.black),
                      child: FittedBox(
                        child: Text(
                          '확인',
                          style: TextStyle(color: Colors.white),
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ] else ...[
              // 일반 유저는 "확인"만
              ElevatedButton(
                onPressed: () => Navigator.pop(context),
                style: ElevatedButton.styleFrom(backgroundColor: Colors.black),
                child: const Text('확인', style: TextStyle(color: Colors.white)),
              ),
            ],
          ],
        ),
      ),
    );
  }
}
