import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'dart:convert';
import 'dart:convert' show utf8;
import 'package:crypto/crypto.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'id_finding_page.dart';
import 'pw_finding_page.dart';
import 'signup_page.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import '../room/main_page.dart';

class LoginPage extends StatefulWidget {
  const LoginPage({Key? key}) : super(key: key);

  @override
  _LoginPageState createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  final TextEditingController idController = TextEditingController();
  final TextEditingController passwordController = TextEditingController();
  bool autoLogin = false;
  String loginErrorMessage = '';

  BannerAd? _bannerAd;
  Widget? _adWidget;

  @override
  void initState() {
    super.initState();
    _bannerAd = BannerAd(
      adUnitId: "ca-app-pub-3151339278746301~1689299887",
      request: const AdRequest(),
      size: AdSize.banner,
      listener: BannerAdListener(
        onAdLoaded: (ad) {
          setState(() {
            _adWidget = AdWidget(ad: ad as AdWithView);
          });
        },
        onAdFailedToLoad: (ad, error) {
          print('Ad failed to load: $error');
          ad.dispose();
        },
      ),
    )..load();
  }

  @override
  void dispose() {
    _bannerAd?.dispose();
    super.dispose();
  }

  Future<void> _login() async {
    String id = idController.text.trim();
    String password = passwordController.text.trim();

    // autoLogin 체크여부
    String autoLoginStatus = autoLogin ? 'Y' : 'N';

    // PW를 sha256으로 해시
    var bytes = utf8.encode(password);
    var digest = sha256.convert(bytes);

    try {
      final response = await http.post(
        Uri.parse('https://eldsoft.com:8097/user/login'),
        headers: {
          'Content-Type': 'application/json',
          'auth_token': '',
        },
        body: jsonEncode({
          'user_id': id,
          'user_pw': digest.toString(),
        }),
      ).timeout(const Duration(seconds: 10));

      // 응답 바디 디코딩
      String responseBody = utf8.decode(response.bodyBytes);

      if (response.statusCode == 200) {
        final Map<String, dynamic> jsonResponse = jsonDecode(responseBody);
        print('jsonResponse: $jsonResponse');

        if (jsonResponse['result'] == 'OK') {
          // 로그인 성공
          final authData = jsonResponse['auth'] ?? {};
          final token = authData['token'] ?? '';
          final userSeq = authData['user_seq'] ?? 0; // 새로 추가

          SharedPreferences prefs = await SharedPreferences.getInstance();
          // 토큰 및 autoLogin 여부 저장
          await prefs.setString('auth_token', token);
          await prefs.setBool('auto_login', autoLogin);
          // (New) 내 user_seq 저장
          await prefs.setInt('my_user_seq', userSeq);

          // 메인 페이지로 이동
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (context) => const MainPage()),
          );
        } else if (jsonResponse['response_info']['msg_title'] == '로그인 실패') {
          // 로그인 실패 메시지
          setState(() {
            loginErrorMessage = '회원정보를 다시 확인해주세요.';
          });
        } else {
          // result != OK 이지만, 다른 이유
          _showDialog('로그인 실패', '서버에서 로그인에 실패했습니다.\n관리자에게 문의해주세요.');
        }
      } else {
        _showDialog('오류', '로그인에 실패했습니다. 관리자에게 문의해주세요.');
      }
    } catch (e) {
      print('로그인 요청 중 오류: $e');
      _showDialog('오류', '로그인 요청이 실패했습니다. 관리자에게 문의해주세요.\n$e');
    }
  }

  void _showDialog(String title, String content) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          backgroundColor: Colors.white,
          title: Text(title, style: const TextStyle(color: Colors.black)),
          content: Text(content, style: const TextStyle(color: Colors.black)),
          actions: <Widget>[
            Center(
              child: TextButton(
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                ),
                child: const Text('확인'),
                onPressed: () {
                  Navigator.of(context).pop();
                },
              ),
            ),
          ],
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        title: const Text('ALL SCORE', style: TextStyle(color: Colors.white)),
        backgroundColor: Colors.black,
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            const Text(
              '로그인',
              style: TextStyle(
                fontSize: 24,
                fontWeight: FontWeight.bold,
                color: Colors.black,
              ),
            ),
            const SizedBox(height: 32),
            TextField(
              controller: idController,
              decoration: InputDecoration(
                labelText: 'ID',
                labelStyle: const TextStyle(color: Colors.black),
                border: const OutlineInputBorder(),
                focusedBorder: const OutlineInputBorder(
                  borderSide: BorderSide(color: Colors.black, width: 2.0),
                ),
              ),
            ),
            const SizedBox(height: 16),
            TextField(
              controller: passwordController,
              obscureText: true,
              decoration: InputDecoration(
                labelText: 'PW',
                labelStyle: const TextStyle(color: Colors.black),
                border: const OutlineInputBorder(),
                focusedBorder: const OutlineInputBorder(
                  borderSide: BorderSide(color: Colors.black, width: 2.0),
                ),
              ),
            ),
            if (loginErrorMessage.isNotEmpty)
              Padding(
                padding: const EdgeInsets.only(top: 8.0),
                child: Text(
                  loginErrorMessage,
                  style: const TextStyle(color: Colors.red),
                ),
              ),
            Row(
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                Checkbox(
                  value: autoLogin,
                  onChanged: (bool? value) {
                    setState(() {
                      autoLogin = value ?? false;
                    });
                  },
                ),
                const Text('자동로그인', style: TextStyle(color: Colors.black)),
              ],
            ),
            const SizedBox(height: 16),
            ElevatedButton(
              onPressed: _login,
              style: ElevatedButton.styleFrom(
                backgroundColor: Colors.black,
                foregroundColor: Colors.white,
              ),
              child: const Text('로그인'),
            ),
            const SizedBox(height: 16),
            TextButton(
              onPressed: () {
                Navigator.push(
                  context,
                  MaterialPageRoute(builder: (context) => const IdFindingPage()),
                );
              },
              child: const Text('ID 찾기', style: TextStyle(color: Colors.black)),
            ),
            TextButton(
              onPressed: () {
                Navigator.push(
                  context,
                  MaterialPageRoute(builder: (context) => const PwFindingPage()),
                );
              },
              child: const Text('PW 찾기', style: TextStyle(color: Colors.black)),
            ),
            TextButton(
              onPressed: () {
                Navigator.push(
                  context,
                  MaterialPageRoute(builder: (context) => const SignUpPage()),
                );
              },
              child: const Text('회원가입', style: TextStyle(color: Colors.black)),
            ),
            const SizedBox(height: 16),
            // 광고 영역
            Container(
              height: 50,
              color: Colors.grey[300],
              child: const Center(child: Text('광고 영역', style: TextStyle(color: Colors.black))),
            ),
          ],
        ),
      ),
    );
  }
}



