import 'package:flutter/material.dart';
import 'package:flutter/services.dart'; // 숫자 입력 제한
import '../../plugins/api.dart';
import '../../dialogs/response_dialog.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'waiting_room_team_page.dart';
import 'waiting_room_private_page.dart';

class CreateRoomPage extends StatefulWidget {
  const CreateRoomPage({Key? key}) : super(key: key);

  @override
  _CreateRoomPageState createState() => _CreateRoomPageState();
}

class _CreateRoomPageState extends State<CreateRoomPage> {
  final TextEditingController _roomNameController = TextEditingController();
  final TextEditingController _roomDescriptionController = TextEditingController();

  /// 공개 여부 (open_yn: 'Y'/'N')
  bool _isPrivate = false;
  final TextEditingController _passwordController = TextEditingController();

  /// 운영시간 (1~6)
  int _selectedHour = 1;

  /// 게임 유형: 개인전 / 팀전
  bool _isTeamGame = false;
  /// 팀 수 (팀전이면 최소 2팀부터)
  int _selectedTeamCount = 2;

  /// 최대 인원
  final TextEditingController _maxParticipantsController = TextEditingController(text: '1');

  /// 점수 공개 범위
  /// - 개인전: 'ALL' / 'PRIVATE'
  /// - 팀전:   'ALL' / 'TEAM' / 'PRIVATE'
  String _selectedScoreOpenRange = 'ALL';  

  @override
  void dispose() {
    _roomNameController.dispose();
    _roomDescriptionController.dispose();
    _passwordController.dispose();
    _maxParticipantsController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      // 전체 흰 배경
      backgroundColor: Colors.white,

      appBar: AppBar(
        title: const Text(
          '방 만들기',
          style: TextStyle(color: Colors.white),
        ),
        backgroundColor: Colors.black,
        elevation: 0,
        leading: IconButton(
          icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
          onPressed: () => Navigator.pop(context),
        ),
      ),

      body: SingleChildScrollView(
        padding: const EdgeInsets.all(16.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            /// (A) 방 제목
            const Text('방 제목',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            const SizedBox(height: 6),
            _buildWhiteBorderTextField(
              controller: _roomNameController,
              hintText: '방 제목을 입력하세요',
            ),
            const SizedBox(height: 16),

            /// (B) 방 소개
            const Text('방 소개',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            const SizedBox(height: 6),
            _buildMultilineBox(
              controller: _roomDescriptionController,
              hintText: '방 소개를 입력하세요',
            ),
            const SizedBox(height: 16),

            /// (C) 비밀번호 설정 (공개 / 비공개)
            const Text('비밀번호 설정',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            Row(
              children: [
                Checkbox(
                  value: !_isPrivate,
                  activeColor: Colors.black,
                  checkColor: Colors.white,
                  onChanged: (value) {
                    setState(() {
                      // 공개 == !비공개
                      _isPrivate = !value!;
                    });
                  },
                ),
                const Text('공개'),
                const SizedBox(width: 10),
                Checkbox(
                  value: _isPrivate,
                  activeColor: Colors.black,
                  checkColor: Colors.white,
                  onChanged: (value) {
                    setState(() {
                      _isPrivate = value!;
                    });
                  },
                ),
                const Text('비공개'),
              ],
            ),
            if (_isPrivate)
              _buildWhiteBorderTextField(
                controller: _passwordController,
                hintText: '비밀번호를 입력하세요',
                obscureText: true,
              ),
            const SizedBox(height: 16),

            /// (D) 운영시간 설정 (1~6시간)
            const Text('운영시간 설정',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            Row(
              children: [
                DropdownButton<int>(
                  value: _selectedHour,
                  dropdownColor: Colors.white,
                  style: const TextStyle(color: Colors.black),
                  items: List.generate(6, (index) => index + 1)
                      .map((value) => DropdownMenuItem<int>(
                            value: value,
                            child: Text(value.toString()),
                          ))
                      .toList(),
                  onChanged: (value) {
                    setState(() {
                      _selectedHour = value!;
                    });
                  },
                ),
                const SizedBox(width: 8),
                const Text('시간'),
              ],
            ),
            const SizedBox(height: 16),

            /// (E) 게임 유형 (개인전/팀전)
            const Text('게임 유형',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            Row(
              children: [
                Checkbox(
                  value: !_isTeamGame,
                  activeColor: Colors.black,
                  checkColor: Colors.white,
                  onChanged: (value) {
                    setState(() {
                      _isTeamGame = !value!;
                      // 점수 공개 범위 초기화
                      _selectedScoreOpenRange = 'ALL';
                    });
                  },
                ),
                const Text('개인전'),
                const SizedBox(width: 10),
                Checkbox(
                  value: _isTeamGame,
                  activeColor: Colors.black,
                  checkColor: Colors.white,
                  onChanged: (value) {
                    setState(() {
                      _isTeamGame = value!;
                      // 점수 공개 범위 초기화
                      _selectedScoreOpenRange = 'ALL';
                    });
                  },
                ),
                const Text('팀전'),
                const SizedBox(width: 16),
                if (_isTeamGame) ...[
                  const Text('팀수: '),
                  const SizedBox(width: 8),
                  // 팀은 최소 2팀부터
                  DropdownButton<int>(
                    value: _selectedTeamCount,
                    dropdownColor: Colors.white,
                    style: const TextStyle(color: Colors.black),
                    items: List.generate(9, (index) => index + 2) // 2 ~ 10
                        .map((value) => DropdownMenuItem<int>(
                              value: value,
                              child: Text(value.toString()),
                            ))
                        .toList(),
                    onChanged: (value) {
                      setState(() {
                        _selectedTeamCount = value!;
                      });
                    },
                  ),
                ],
              ],
            ),
            const SizedBox(height: 16),

            /// (F) 최대 인원수
            const Text('최대 인원수',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            Row(
              children: [
                SizedBox(
                  width: 80,
                  child: TextField(
                    controller: _maxParticipantsController,
                    keyboardType: TextInputType.number,
                    textAlign: TextAlign.center,
                    style: const TextStyle(fontSize: 16),
                    decoration: const InputDecoration(
                      border: OutlineInputBorder(),
                      contentPadding: EdgeInsets.symmetric(vertical: 8),
                    ),
                    inputFormatters: [
                      FilteringTextInputFormatter.digitsOnly,
                      LengthLimitingTextInputFormatter(3),
                    ],
                    onChanged: (value) {
                      if (value.isNotEmpty) {
                        final number = int.tryParse(value);
                        if (number == null || number < 1 || number > 100) {
                          _maxParticipantsController.text = '1';
                          _maxParticipantsController.selection =
                              TextSelection.fromPosition(
                            TextPosition(
                              offset: _maxParticipantsController.text.length,
                            ),
                          );
                        }
                      }
                    },
                  ),
                ),
                const SizedBox(width: 8),
                const Text('명', style: TextStyle(fontSize: 16)),
              ],
            ),
            const SizedBox(height: 16),

            /// (G) 점수 공개 범위
            ///  개인전: 'ALL', 'PRIVATE'
            ///  팀전:   'ALL', 'TEAM', 'PRIVATE'
            const Text(
              '점수 공개 범위 설정',
              style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
            ),
            const SizedBox(height: 8),

            if (!_isTeamGame) 
              // 개인전 → ALL, PRIVATE
              Column(
                children: [
                  RadioListTile<String>(
                    title: const Text('전체'),
                    value: 'ALL',
                    groupValue: _selectedScoreOpenRange,
                    activeColor: Colors.black,
                    onChanged: (value) {
                      setState(() {
                        _selectedScoreOpenRange = value!;
                      });
                    },
                  ),
                  RadioListTile<String>(
                    title: const Text('개인'),
                    value: 'PRIVATE',
                    groupValue: _selectedScoreOpenRange,
                    activeColor: Colors.black,
                    onChanged: (value) {
                      setState(() {
                        _selectedScoreOpenRange = value!;
                      });
                    },
                  ),
                ],
              )
            else
              // 팀전 → ALL, TEAM, PRIVATE
              Column(
                children: [
                  RadioListTile<String>(
                    title: const Text('전체'),
                    value: 'ALL',
                    groupValue: _selectedScoreOpenRange,
                    activeColor: Colors.black,
                    onChanged: (value) {
                      setState(() {
                        _selectedScoreOpenRange = value!;
                      });
                    },
                  ),
                  RadioListTile<String>(
                    title: const Text('팀'),
                    value: 'TEAM',
                    groupValue: _selectedScoreOpenRange,
                    activeColor: Colors.black,
                    onChanged: (value) {
                      setState(() {
                        _selectedScoreOpenRange = value!;
                      });
                    },
                  ),
                  RadioListTile<String>(
                    title: const Text('개인'),
                    value: 'PRIVATE',
                    groupValue: _selectedScoreOpenRange,
                    activeColor: Colors.black,
                    onChanged: (value) {
                      setState(() {
                        _selectedScoreOpenRange = value!;
                      });
                    },
                  ),
                ],
              ),
            const SizedBox(height: 24),

            /// (H) "방 생성하기" 버튼
            Center(
              child: ElevatedButton(
                onPressed: () async {
                  try {
                    final serverResponse = await createRoom(); 
                    if (serverResponse['result'] == 'OK') {
                      final serverResponse1 = serverResponse['response'];
                      if (serverResponse1['result'] == 'OK') {
                        // 성공 응답시 roomSeq 저장
                        final roomSeq = serverResponse1['data']['room_seq'];

                        // 방 생성 성공 → 대기 방으로 이동
                        if (_isTeamGame) {
                          Navigator.pushReplacement(
                            context,
                            MaterialPageRoute(
                              builder: (_) => WaitingRoomTeamPage(
                                roomSeq: roomSeq,
                                roomType: 'team',
                              ),
                            ),
                          );
                        } else {
                          Navigator.pushReplacement(
                            context,
                            MaterialPageRoute(
                              builder: (_) => WaitingRoomPrivatePage(
                                roomSeq: roomSeq,
                                roomType: 'private',
                              ),
                            ),
                          );
                        }
                      } else {
                        showResponseDialog(
                          context,
                          '${serverResponse1['response_info']['msg_title']}',
                          '${serverResponse1['response_info']['msg_content']}',
                        );
                      }
                    } else {
                      showResponseDialog(
                        context,
                        '방 생성 실패',
                        '서버에 문제가 있습니다. 관리자에게 문의해주세요.',
                      );
                    }
                  } catch (e) {
                    showResponseDialog(context, '방 생성 실패', e.toString());
                  }
                },
                style: ElevatedButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                  padding: const EdgeInsets.symmetric(vertical: 12, horizontal: 40),
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(40),
                  ),
                ),
                child: const Text('방 생성하기', style: TextStyle(fontSize: 16)),
              ),
            ),
            const SizedBox(height: 16),
          ],
        ),
      ),
    );
  }

  /// “방 생성” 로직
  Future<Map<String, dynamic>> createRoom() async {
    // requestBody 생성
    final requestBody = {
      'room_title': _roomNameController.text,
      'room_intro': _roomDescriptionController.text,
      'open_yn': _isPrivate ? 'N' : 'Y',
      'room_pw': _isPrivate ? _passwordController.text : '',
      'running_time': _selectedHour.toString(),
      'room_type': _isTeamGame ? 'team' : 'private',
      'number_of_teams': _selectedTeamCount.toString(),
      'number_of_people': _maxParticipantsController.text,
      // 점수공개범위:
      //  개인전: 'ALL', 'PRIVATE'
      //  팀전:   'ALL', 'TEAM', 'PRIVATE'
      'score_open_range': _selectedScoreOpenRange,

      'room_status': 'WAIT',
    };

    try {
      final serverResponse =
          await Api.serverRequest(uri: '/room/score/create/room', body: requestBody);

      if (serverResponse == null) {
        throw Exception('서버 응답이 null입니다.');
      }
      if (serverResponse['result'] == 'OK') {
        return serverResponse;
      } else {
        return {'result': 'FAIL'};
      }
    } catch (e) {
      print('serverResponse 오류: $e');
      return {'result': 'FAIL'};
    }
  }

  /// 테두리 + 흰 배경 TextField
  Widget _buildWhiteBorderTextField({
    required TextEditingController controller,
    String hintText = '',
    bool obscureText = false,
  }) {
    return TextField(
      controller: controller,
      obscureText: obscureText,
      style: const TextStyle(color: Colors.black),
      decoration: InputDecoration(
        hintText: hintText,
        hintStyle: TextStyle(color: Colors.grey.shade400),
        border: const OutlineInputBorder(),
        filled: true,
        fillColor: Colors.white,
      ),
    );
  }

  /// 다중라인 텍스트박스 (방 소개 등)
  Widget _buildMultilineBox({
    required TextEditingController controller,
    String hintText = '',
  }) {
    return Container(
      padding: const EdgeInsets.all(8),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: TextField(
        controller: controller,
        maxLines: 4,
        style: const TextStyle(color: Colors.black),
        decoration: InputDecoration(
          border: InputBorder.none,
          hintText: hintText,
          hintStyle: TextStyle(color: Colors.grey.shade400),
        ),
      ),
    );
  }
}
