import 'package:flutter/material.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'main_page.dart';
import 'finish_private_page.dart';   // (★) 개인전 종료화면
import '../../plugins/api.dart';
import '../../dialogs/response_dialog.dart';

// 점수 수정 모달
import '../../dialogs/score_edit_dialog.dart';
// 기존 사용자 정보 모달 (관리자/방장X)
import '../../dialogs/user_info_basic_dialog.dart';

class PlayingPrivatePage extends StatefulWidget {
  final int roomSeq;
  final String roomTitle;

  const PlayingPrivatePage({
    Key? key,
    required this.roomSeq,
    required this.roomTitle,
  }) : super(key: key);

  @override
  State<PlayingPrivatePage> createState() => _PlayingPrivatePageState();
}

class _PlayingPrivatePageState extends State<PlayingPrivatePage> {
  // FRD
  late DatabaseReference _roomRef;
  Stream<DatabaseEvent>? _roomStream;

  String roomMasterYn = 'N';
  String roomTitle = '';

  int myScore = 0;

  // (ADMIN 제외) 플레이어 목록
  List<Map<String, dynamic>> _scoreList = [];

  bool _isLoading = true;

  // 내 user_seq
  String mySeq = '0';

  // userListMap
  Map<String, bool> _userListMap = {};

  @override
  void initState() {
    super.initState();
    roomTitle = widget.roomTitle;
    _initFirebase();
  }

  Future<void> _initFirebase() async {
    final prefs = await SharedPreferences.getInstance();
    mySeq = prefs.getInt('my_user_seq')?.toString() ?? '0';

    final roomKey = 'korea-${widget.roomSeq}';
    _roomRef = FirebaseDatabase.instance.ref('rooms/$roomKey');

    _listenRoomData();
  }

  void _listenRoomData() {
    _roomStream = _roomRef.onValue;
    _roomStream?.listen((event) {
      final snapshot = event.snapshot;
      if (!snapshot.exists) {
        setState(() {
          _isLoading = false;
          roomTitle = '방 정보 없음';
          _scoreList = [];
          myScore = 0;
        });
        return;
      }

      final data = snapshot.value as Map<dynamic, dynamic>? ?? {};

      final roomInfoData = data['roomInfo'] as Map<dynamic, dynamic>? ?? {};
      final userInfoData = data['userInfo'] as Map<dynamic, dynamic>? ?? {};
      final userListData = data['userList'] as Map<dynamic, dynamic>?;

      // 방 상태 체크
      final roomStatus = (roomInfoData['room_status'] ?? 'WAIT').toString().toUpperCase();

      // 만약 FINISH라면 => 종료 페이지 이동
      if (roomStatus == 'FINISH') {
        // 모든 유저 -> 종료 페이지
        // (중복 이동 방지)
        if (mounted) {
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (_) => FinishPrivatePage(roomSeq: widget.roomSeq)),
          );
        }
        return;
      }

      setState(() {
        // 방장 여부
        final masterSeq = roomInfoData['master_user_seq'];
        roomMasterYn = (masterSeq != null && masterSeq.toString() == mySeq) ? 'Y' : 'N';

        // 방 제목
        final newTitle = (roomInfoData['room_title'] ?? '') as String;
        if (newTitle.isNotEmpty) roomTitle = newTitle;

        // userListMap
        _userListMap.clear();
        if (userListData != null) {
          userListData.forEach((k, v) {
            _userListMap[k.toString()] = (v == true);
          });
        }

        // 전체 참가자
        final List<Map<String, dynamic>> rawList = [];
        userInfoData.forEach((uSeq, uData) {
          rawList.add({
            'user_seq': uSeq,
            'participant_type': (uData['participant_type'] ?? '').toString().toUpperCase(),
            'nickname': uData['nickname'] ?? '유저',
            'score': uData['score'] ?? 0,
            'profile_img': uData['profile_img'] ?? '',
            'department': uData['department'] ?? '',
            'introduce_myself': uData['introduce_myself'] ?? '',
            'is_my_score': (uSeq.toString() == mySeq) ? 'Y' : 'N',
          });
        });

        // 내 점수
        int tempMyScore = 0;
        for (var u in rawList) {
          if ((u['is_my_score'] ?? 'N') == 'Y') {
            tempMyScore = u['score'] ?? 0;
          }
        }

        // ADMIN 제외
        final playerList = rawList.where((u) => u['participant_type'] != 'ADMIN').toList();
        // 점수 내림차순
        playerList.sort((a, b) {
          final scoreA = a['score'] ?? 0;
          final scoreB = b['score'] ?? 0;
          return scoreB.compareTo(scoreA);
        });

        myScore = tempMyScore;
        _scoreList = playerList;
        _isLoading = false;
      });
    }, onError: (err) {
      setState(() {
        _isLoading = false;
        roomTitle = '오류 발생';
      });
    });
  }

  /// (A) WillPopScope + AppBar leading
  Future<bool> _onBackPressed() async {
    // 방장? => 게임 종료 API
    if (roomMasterYn == 'Y') {
      await _requestFinish(); 
    }

    // userList => false
    final userRef = _roomRef.child('userList').child(mySeq);
    await userRef.set(false);

    if (!mounted) return false;
    Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
    return false;
  }

  /// (B) 서버에 "게임 종료" 요청
  Future<void> _requestFinish() async {
    final reqBody = {
      "room_seq": "${widget.roomSeq}",
      "room_type": "PRIVATE",
    };
    try {
      final resp = await Api.serverRequest(
        uri: '/room/score/game/finish',
        body: reqBody,
      );
      // OK / FAIL 등은 여기서 특별 처리 없이 넘어감
      // room_status = FINISH => FRD에서 반영 -> 모든 참여자 이동
    } catch (e) {
      // 무시하거나 모달 표시
      print('게임 종료 API 에러: $e');
    }
  }

  /// (C) 각 참가자 표시
  Widget _buildScoreItem(Map<String, dynamic> user) {
    final userSeq = user['user_seq'].toString();
    final score = user['score'] ?? 0;
    final nickname = user['nickname'] ?? '유저';

    final bool isActive = _userListMap[userSeq] ?? true; 
    final hasExited = !isActive;

    return GestureDetector(
      onTap: () => _onUserTapped(user),
      child: Container(
        width: 60,
        margin: const EdgeInsets.all(4),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            hasExited
                ? Text('X', style: TextStyle(fontSize: 20, color: Colors.redAccent, fontWeight: FontWeight.bold))
                : Text('$score', style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold, color: Colors.black)),
            const SizedBox(height: 2),
            Container(
              width: 30,
              height: 30,
              decoration: BoxDecoration(
                shape: BoxShape.circle,
                border: Border.all(color: hasExited ? Colors.redAccent : Colors.black),
              ),
              child: hasExited
                  ? Center(
                      child: Text('X', style: TextStyle(fontSize: 14, color: Colors.redAccent, fontWeight: FontWeight.bold)),
                    )
                  : ClipOval(
                      child: Image.network(
                        'https://eldsoft.com:8097/images${user['profile_img']}',
                        fit: BoxFit.cover,
                        errorBuilder: (ctx, err, st) => const Center(
                          child: Text('ERR', style: TextStyle(fontSize: 8, color: Colors.black)),
                        ),
                      ),
                    ),
            ),
            const SizedBox(height: 2),
            Text(
              nickname,
              style: TextStyle(fontSize: 11, color: hasExited ? Colors.redAccent : Colors.black),
              overflow: TextOverflow.ellipsis,
            ),
          ],
        ),
      ),
    );
  }

  Future<void> _onUserTapped(Map<String, dynamic> userData) async {
    final pType = (userData['participant_type'] ?? '').toString().toUpperCase();
    if (pType == 'ADMIN') {
      // 점수 수정 모달
      await showDialog(
        context: context,
        barrierDismissible: false,
        builder: (_) => ScoreEditDialog(
          roomSeq: widget.roomSeq,
          roomType: 'PRIVATE',
          userData: userData,
        ),
      );
    } else if (roomMasterYn == 'Y') {
      // 방장(PLAYER)도 점수 수정
      await showDialog(
        context: context,
        barrierDismissible: false,
        builder: (_) => ScoreEditDialog(
          roomSeq: widget.roomSeq,
          roomType: 'PRIVATE',
          userData: userData,
        ),
      );
    } else {
      // 일반 모달
      await showDialog(
        context: context,
        barrierDismissible: false,
        builder: (_) => UserInfoBasicDialog(userData: userData),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: _onBackPressed,
      child: Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          backgroundColor: Colors.black,
          elevation: 0,
          leading: IconButton(
            icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
            onPressed: _onBackPressed,
          ),
          title: Text(
            roomTitle.isNotEmpty ? roomTitle : '진행중 (개인전)',
            style: const TextStyle(color: Colors.white),
          ),
          actions: [
            if (roomMasterYn == 'Y')
              TextButton(
                onPressed: () async {
                  // 방장 수동 종료버튼
                  await _requestFinish();
                },
                child: const Text('게임종료', style: TextStyle(color: Colors.white)),
              ),
          ],
        ),
        body: _isLoading
            ? const Center(child: CircularProgressIndicator())
            : Column(
                children: [
                  // 내 점수
                  Container(
                    width: double.infinity,
                    color: Colors.white,
                    padding: const EdgeInsets.symmetric(vertical: 16),
                    child: Column(
                      children: [
                        const Text('내 점수', style: TextStyle(fontSize: 14, fontWeight: FontWeight.bold, color: Colors.black)),
                        const SizedBox(height: 4),
                        Text('$myScore', style: const TextStyle(fontSize: 36, fontWeight: FontWeight.bold, color: Colors.black)),
                      ],
                    ),
                  ),
                  const Divider(height: 1, color: Colors.black),

                  Expanded(
                    child: Container(
                      padding: const EdgeInsets.all(8),
                      child: SingleChildScrollView(
                        child: Wrap(
                          spacing: 8,
                          runSpacing: 8,
                          children: _scoreList.map(_buildScoreItem).toList(),
                        ),
                      ),
                    ),
                  ),

                  Container(
                    height: 50,
                    decoration: BoxDecoration(
                      color: Colors.white,
                      border: Border.all(color: Colors.black, width: 1),
                    ),
                    child: const Center(
                      child: Text('구글 광고', style: TextStyle(color: Colors.black)),
                    ),
                  ),
                ],
              ),
      ),
    );
  }
}
