import 'package:flutter/material.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'main_page.dart';
import 'finish_team_page.dart';  // (★) 팀전 종료화면
import '../../plugins/api.dart';
import '../../dialogs/response_dialog.dart';
import '../../dialogs/score_edit_dialog.dart';
import '../../dialogs/user_info_basic_dialog.dart';

class PlayingTeamPage extends StatefulWidget {
  final int roomSeq;
  final String roomTitle;

  const PlayingTeamPage({
    Key? key,
    required this.roomSeq,
    required this.roomTitle,
  }) : super(key: key);

  @override
  State<PlayingTeamPage> createState() => _PlayingTeamPageState();
}

class _PlayingTeamPageState extends State<PlayingTeamPage> {
  late DatabaseReference _roomRef;
  Stream<DatabaseEvent>? _roomStream;

  String roomMasterYn = 'N'; 
  String roomTitle = '';

  int myScore = 0;
  int myTeamScore = 0;

  Map<String, int> _teamScoreMap = {};
  Map<String, List<Map<String, dynamic>>> _teamMap = {};

  bool _isLoading = true;

  String mySeq = '0';

  // userListMap
  Map<String, bool> _userListMap = {};

  @override
  void initState() {
    super.initState();
    roomTitle = widget.roomTitle;
    _initFirebase();
  }

  Future<void> _initFirebase() async {
    final prefs = await SharedPreferences.getInstance();
    mySeq = prefs.getInt('my_user_seq')?.toString() ?? '0';

    final roomKey = 'korea-${widget.roomSeq}';
    _roomRef = FirebaseDatabase.instance.ref('rooms/$roomKey');

    _listenRoomData();
  }

  void _listenRoomData() {
    _roomStream = _roomRef.onValue;
    _roomStream?.listen((event) {
      final snapshot = event.snapshot;
      if (!snapshot.exists) {
        setState(() {
          _isLoading = false;
          roomTitle = '방 정보 없음';
          myScore = 0;
          myTeamScore = 0;
          _teamScoreMap = {};
          _teamMap = {};
        });
        return;
      }

      final data = snapshot.value as Map<dynamic, dynamic>? ?? {};

      final roomInfoData = data['roomInfo'] as Map<dynamic, dynamic>? ?? {};
      final userInfoData = data['userInfo'] as Map<dynamic, dynamic>? ?? {};
      final userListData = data['userList'] as Map<dynamic, dynamic>?;

      // room_status
      final roomStatus = (roomInfoData['room_status'] ?? 'WAIT').toString().toUpperCase();

      // FINISH -> 종료화면
      if (roomStatus == 'FINISH') {
        if (mounted) {
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (_) => FinishTeamPage(roomSeq: widget.roomSeq)),
          );
        }
        return;
      }

      setState(() {
        // 방장 여부
        final masterSeq = roomInfoData['master_user_seq'];
        roomMasterYn = (masterSeq != null && masterSeq.toString() == mySeq) ? 'Y' : 'N';

        final newTitle = (roomInfoData['room_title'] ?? '') as String;
        if (newTitle.isNotEmpty) roomTitle = newTitle;

        // userListMap
        _userListMap.clear();
        if (userListData != null) {
          userListData.forEach((k, v) {
            _userListMap[k.toString()] = (v == true);
          });
        }

        // 전체 유저
        final List<Map<String, dynamic>> rawList = [];
        userInfoData.forEach((uSeq, uData) {
          rawList.add({
            'user_seq': uSeq,
            'participant_type': (uData['participant_type'] ?? '').toString().toUpperCase(),
            'nickname': uData['nickname'] ?? '유저',
            'team_name': (uData['team_name'] ?? '').toString().toUpperCase(),
            'score': uData['score'] ?? 0,
          });
        });

        // 내 점수/팀점수
        int tmpMyScore = 0;
        int tmpMyTeamScore = 0;
        String myTeam = 'WAIT';

        for (var user in rawList) {
          final uSeq = user['user_seq'].toString();
          final sc = (user['score'] ?? 0) as int;
          final tName = user['team_name'] ?? 'WAIT';
          if (uSeq == mySeq) {
            tmpMyScore = sc;
            myTeam = tName;
          }
        }

        // 내 팀 점수
        for (var user in rawList) {
          final tName = user['team_name'] ?? 'WAIT';
          final sc = (user['score'] ?? 0) as int;
          if (tName == myTeam && tName != 'WAIT') {
            tmpMyTeamScore += sc;
          }
        }

        // 팀별 분류 (ADMIN/WAIT 제외)
        final Map<String, List<Map<String, dynamic>>> tMap = {};
        final Map<String, int> tScoreMap = {};

        for (var user in rawList) {
          final pType = user['participant_type'];
          final tName = user['team_name'] ?? 'WAIT';
          if (pType == 'ADMIN') continue;
          if (tName == 'WAIT') continue;

          tMap.putIfAbsent(tName, () => []);
          tMap[tName]!.add(user);
        }

        // 팀 점수 합
        tMap.forEach((k, members) {
          int sumScore = 0;
          for (var m in members) {
            sumScore += (m['score'] ?? 0) as int;
          }
          tScoreMap[k] = sumScore;
        });

        myScore = tmpMyScore;
        myTeamScore = tmpMyTeamScore;
        _teamMap = tMap;
        _teamScoreMap = tScoreMap;
        _isLoading = false;
      });
    }, onError: (err) {
      setState(() {
        _isLoading = false;
        roomTitle = '오류 발생';
      });
    });
  }

  /// (A) 뒤로가기 -> 방장? => Finish API
  Future<bool> _onBackPressed() async {
    if (roomMasterYn == 'Y') {
      await _requestFinish();
    }
    // userList => false
    final userRef = _roomRef.child('userList').child(mySeq);
    await userRef.set(false);

    if (!mounted) return false;
    Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
    return false;
  }

  Future<void> _requestFinish() async {
    final body = {
      "room_seq": "${widget.roomSeq}",
      "room_type": "TEAM",
    };
    try {
      final resp = await Api.serverRequest(
        uri: '/room/score/game/finish',
        body: body,
      );
      // result ...
    } catch (e) {
      print('finish API error: $e');
    }
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: _onBackPressed,
      child: Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          title: Text(
            roomTitle.isNotEmpty ? roomTitle : '진행중 (팀전)',
            style: const TextStyle(color: Colors.white),
          ),
          backgroundColor: Colors.black,
          elevation: 0,
          leading: IconButton(
            icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
            onPressed: _onBackPressed,
          ),
          actions: [
            if (roomMasterYn == 'Y')
              TextButton(
                onPressed: _requestFinish,
                child: const Text('게임종료', style: TextStyle(color: Colors.white)),
              ),
          ],
        ),
        body: _isLoading
            ? const Center(child: CircularProgressIndicator())
            : Column(
                children: [
                  // 내 점수 / 팀 점수
                  Container(
                    color: Colors.white,
                    padding: const EdgeInsets.only(top: 16, bottom: 16),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      children: [
                        Column(
                          children: [
                            const Text('내 점수', style: TextStyle(fontSize: 14, fontWeight: FontWeight.bold, color: Colors.black)),
                            const SizedBox(height: 4),
                            Text('$myScore', style: const TextStyle(fontSize: 36, fontWeight: FontWeight.bold, color: Colors.black)),
                          ],
                        ),
                        Container(width: 1, height: 60, color: Colors.black),
                        Column(
                          children: [
                            const Text('우리 팀 점수', style: TextStyle(fontSize: 14, fontWeight: FontWeight.bold, color: Colors.black)),
                            const SizedBox(height: 4),
                            Text('$myTeamScore', style: const TextStyle(fontSize: 36, fontWeight: FontWeight.bold, color: Colors.black)),
                          ],
                        ),
                      ],
                    ),
                  ),
                  const Divider(height: 1, color: Colors.black),

                  Expanded(
                    child: SingleChildScrollView(
                      padding: const EdgeInsets.all(8),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: _teamMap.keys.map(_buildTeamSection).toList(),
                      ),
                    ),
                  ),
                  Container(
                    height: 50,
                    decoration: BoxDecoration(
                      color: Colors.white,
                      border: Border.all(color: Colors.black, width: 1),
                    ),
                    child: const Center(
                      child: Text('구글 광고', style: TextStyle(color: Colors.black)),
                    ),
                  ),
                ],
              ),
      ),
    );
  }

  Widget _buildTeamSection(String teamName) {
    final upperName = teamName.toUpperCase();
    final members = _teamMap[upperName] ?? [];
    final teamScore = _teamScoreMap[upperName] ?? 0;

    return Container(
      width: double.infinity,
      margin: const EdgeInsets.only(bottom: 8),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: Column(
        children: [
          Container(
            color: Colors.black,
            width: double.infinity,
            padding: const EdgeInsets.symmetric(vertical: 8),
            child: Center(
              child: Text('$teamName (팀점수 $teamScore)', style: const TextStyle(color: Colors.white, fontWeight: FontWeight.bold)),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(8),
            child: SingleChildScrollView(
              scrollDirection: Axis.horizontal,
              child: Row(children: members.map(_buildTeamMemberItem).toList()),
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildTeamMemberItem(Map<String, dynamic> userData) {
    final userSeq = userData['user_seq'].toString();
    final score = userData['score'] ?? 0;
    final nickname = userData['nickname'] ?? '유저';

    final bool isActive = _userListMap[userSeq] ?? true;
    final hasExited = !isActive;

    return GestureDetector(
      onTap: () => _onUserTapped(userData),
      child: Container(
        width: 60,
        margin: const EdgeInsets.symmetric(horizontal: 6, vertical: 4),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            hasExited
                ? Text('X', style: TextStyle(fontSize: 18, color: Colors.redAccent, fontWeight: FontWeight.bold))
                : Text('$score', style: const TextStyle(fontSize: 18, fontWeight: FontWeight.bold, color: Colors.black)),
            const SizedBox(height: 2),
            Container(
              width: 30,
              height: 30,
              decoration: BoxDecoration(
                shape: BoxShape.circle,
                border: Border.all(color: hasExited ? Colors.redAccent : Colors.black),
              ),
              child: hasExited
                  ? Center(child: Text('X', style: TextStyle(fontSize: 14, color: Colors.redAccent, fontWeight: FontWeight.bold)))
                  : ClipOval(
                      child: Image.network(
                        'https://eldsoft.com:8097/images${userData['profile_img']}',
                        fit: BoxFit.cover,
                        errorBuilder: (ctx, err, st) => const Center(child: Text('ERR', style: TextStyle(fontSize: 8, color: Colors.black))),
                      ),
                    ),
            ),
            const SizedBox(height: 2),
            Text(
              nickname,
              style: TextStyle(fontSize: 11, color: hasExited ? Colors.redAccent : Colors.black),
              overflow: TextOverflow.ellipsis,
            ),
          ],
        ),
      ),
    );
  }

  Future<void> _onUserTapped(Map<String, dynamic> userData) async {
    final pType = (userData['participant_type'] ?? '').toString().toUpperCase();
    if (pType == 'ADMIN') {
      // 점수 수정
      await showDialog(
        context: context,
        barrierDismissible: false,
        builder: (_) => ScoreEditDialog(
          roomSeq: widget.roomSeq,
          roomType: 'TEAM',
          userData: userData,
        ),
      );
    } else if (roomMasterYn == 'Y') {
      await showDialog(
        context: context,
        barrierDismissible: false,
        builder: (_) => ScoreEditDialog(
          roomSeq: widget.roomSeq,
          roomType: 'TEAM',
          userData: userData,
        ),
      );
    } else {
      await showDialog(
        context: context,
        barrierDismissible: false,
        builder: (_) => UserInfoBasicDialog(userData: userData),
      );
    }
  }
}
