import 'package:flutter/material.dart';
import 'dart:async';

import '../../plugins/api.dart';            // 서버 요청용 (예: Api.serverRequest)
import '../../dialogs/response_dialog.dart'; // 모달창 띄우기 예시
import '../../dialogs/room_detail_dialog.dart'; // 분리된 모달창 import

/// 서버로부터 방 리스트를 검색/조회하는 페이지
/// - roomStatus: "WAIT"/"RUNNING"/"FINISH"
/// - 1페이지당 10개씩 로드, 스크롤 최하단 도달 시 다음 페이지 자동 로드
/// - 검색창을 통해 room_title 필터링
class RoomSearchListPage extends StatefulWidget {
  final String roomStatus; // WAIT / RUNNING / FINISH

  const RoomSearchListPage({Key? key, required this.roomStatus}) : super(key: key);

  @override
  State<RoomSearchListPage> createState() => _RoomSearchListPageState();
}

class _RoomSearchListPageState extends State<RoomSearchListPage> {
  final TextEditingController _searchController = TextEditingController();

  // 방 목록
  List<Map<String, dynamic>> _roomList = [];

  bool _isLoading = false;
  bool _hasMore = true;
  int _currentPage = 1;
  final int _pageSize = 10;

  late ScrollController _scrollController;

  @override
  void initState() {
    super.initState();

    _scrollController = ScrollController()..addListener(_onScroll);
    _fetchRoomList(isRefresh: true);
  }

  @override
  void dispose() {
    _scrollController.removeListener(_onScroll);
    _scrollController.dispose();
    _searchController.dispose();
    super.dispose();
  }

  /// 스크롤이 최하단 근처 도달 시 다음 페이지 로드
  void _onScroll() {
    if (!_scrollController.hasClients) return;
    final thresholdPixels = 200;
    final maxScroll = _scrollController.position.maxScrollExtent;
    final currentScroll = _scrollController.position.pixels;

    if (maxScroll - currentScroll <= thresholdPixels) {
      _fetchRoomList(isRefresh: false);
    }
  }

  /// (1) 서버에서 방 리스트 가져오기
  Future<void> _fetchRoomList({required bool isRefresh}) async {
    if (_isLoading) return; 
    if (!isRefresh && !_hasMore) return;

    setState(() => _isLoading = true);

    if (isRefresh) {
      _currentPage = 1;
      _hasMore = true;
      _roomList.clear();
    }

    // 서버 API 요구사항에 맞춰 WAIT/RUNNING/FINISH (대문자) 사용
    final String searchType = widget.roomStatus.toUpperCase();
    final String searchValue = _searchController.text.trim();
    final String searchPage = _currentPage.toString();

    final requestBody = {
      "search_type": searchType,   
      "search_value": searchValue,
      "search_page": searchPage,
    };

    try {
      final response = await Api.serverRequest(
        uri: '/room/score/room/list',
        body: requestBody,
      );

      print('🔵 response: $response');

      // (참고) 서버 구조: { result: OK, response: {...}, ... }
      if (response == null || response['result'] != 'OK') {
        showResponseDialog(context, '오류', '방 목록을 불러오지 못했습니다.');
      } else {
        final innerResp = response['response'];
        if (innerResp == null || innerResp['result'] != 'OK') {
          showResponseDialog(context, '오류', '내부 응답이 잘못되었습니다.');
        } else {
          final respData = innerResp['data'];
          if (respData is List) {
            if (respData.isEmpty) {
              _hasMore = false;
            } else {
              for (var item in respData) {
                print('🔵 item: $item');
                final parsedItem = {
                  'room_seq': item['room_seq'] ?? 0,
                  'nickname': item['nickname'] ?? '사용자',
                  'room_status': _statusToKr(item['room_status'] ?? ''),
                  'open_yn': (item['open_yn'] == 'Y') ? '공개' : '비공개',
                  'room_type': item['room_type_name'] ?? 'private',
                  'room_title': item['room_title'] ?? '(방제목 없음)',
                  'room_intro': item['room_intro'] ?? '',
                  'now_people': item['now_number_of_people']?.toString() ?? '0',
                  'max_people': item['number_of_people']?.toString() ?? '0',
                  'start_dt': item['start_dt'],
                  'end_dt': item['end_dt'],
                };
                _roomList.add(parsedItem);
              }

              if (respData.length < _pageSize) {
                _hasMore = false;
              }
              _currentPage++;
            }
          }
        }
      }
    } catch (e) {
      showResponseDialog(context, '오류', '서버 요청 중 예외 발생: $e');
    } finally {
      setState(() => _isLoading = false);
    }
  }

  /// WAIT->'대기중', RUNNING->'진행중', FINISH->'종료'
  String _statusToKr(String status) {
    switch (status.toUpperCase()) {
      case 'WAIT':
        return '대기중';
      case 'RUNNING':
        return '진행중';
      case 'FINISH':
        return '종료';
      default:
        return status;
    }
  }

  void _onSearch() {
    _fetchRoomList(isRefresh: true);
  }

  void _onRoomItemTap(Map<String, dynamic> item) {
    // 여기서 분리된 모달 호출
    showDialog(
      context: context,
      barrierDismissible: false,
      builder: (_) => RoomDetailDialog(roomData: item),
    );
  }

  @override
  Widget build(BuildContext context) {
    final statusKr = _statusToKr(widget.roomStatus);

    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        backgroundColor: Colors.black,
        title: Text('$statusKr 방 검색', style: const TextStyle(color: Colors.white)),
        leading: IconButton(
          icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
          onPressed: () => Navigator.pop(context),
        ),
      ),
      body: Column(
        children: [
          // (A) 검색창
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: TextField(
              controller: _searchController,
              onSubmitted: (_) => _onSearch(),
              style: const TextStyle(color: Colors.black),
              decoration: InputDecoration(
                hintText: '방 제목 입력',
                hintStyle: TextStyle(color: Colors.grey.shade400),
                prefixIcon: const Icon(Icons.search, color: Colors.black54),
                suffixIcon: IconButton(
                  icon: const Icon(Icons.close, color: Colors.black54),
                  onPressed: () {
                    _searchController.clear();
                  },
                ),
                border: OutlineInputBorder(
                  borderSide: const BorderSide(color: Colors.black),
                  borderRadius: BorderRadius.circular(20),
                ),
                contentPadding: const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
              ),
            ),
          ),

          // (B) 로딩 표시 or 리스트
          Expanded(
            child: _isLoading && _roomList.isEmpty
                ? const Center(child: CircularProgressIndicator())
                : _buildRoomListView(),
          ),

          // (C) 하단 광고
          Container(
            height: 60,
            color: Colors.white,
            child: Center(
              child: Container(
                height: 50,
                width: 300,
                color: Colors.grey.shade400,
                child: const Center(
                  child: Text('구글 광고', style: TextStyle(color: Colors.black)),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildRoomListView() {
    print('🔵 _roomList: $_roomList');
    if (_roomList.isEmpty) {
      return const Center(
        child: Text(
          '검색 결과가 없습니다.',
          style: TextStyle(color: Colors.black),
        ),
      );
    }

    return ListView.builder(
      controller: _scrollController,
      padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 16),
      itemCount: _roomList.length,
      itemBuilder: (context, index) {
        final item = _roomList[index];
        return _buildRoomItem(item);
      },
    );
  }

  Widget _buildRoomItem(Map<String, dynamic> item) {
    final roomTitle  = item['room_title'] ?? '(방제목 없음)';
    final nickname   = item['nickname'] ?? '유저';
    final roomStatus = item['room_status'] ?? '대기중';
    final openYn     = item['open_yn'] ?? '공개';
    final nowPeople  = item['now_people'] ?? '0';
    final maxPeople  = item['max_people'] ?? '0';
    final roomIntro  = item['room_intro'] ?? '';

    return GestureDetector(
      onTap: () => _onRoomItemTap(item),
      child: Container(
        margin: const EdgeInsets.only(bottom: 12),
        padding: const EdgeInsets.all(10),
        decoration: BoxDecoration(
          border: Border.all(color: Colors.black54),
          borderRadius: BorderRadius.circular(8),
        ),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Text(roomTitle, style: const TextStyle(fontSize: 16, fontWeight: FontWeight.bold)),
            const SizedBox(height: 4),
            Text('$nickname / $roomStatus / $openYn / $nowPeople/$maxPeople명'),
            const SizedBox(height: 4),
            Text(roomIntro, style: const TextStyle(fontSize: 12)),
          ],
        ),
      ),
    );
  }
}
