import 'package:flutter/material.dart';
import 'dart:async';
import 'package:firebase_database/firebase_database.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'main_page.dart';
import '../../plugins/api.dart';         // 서버 API
import '../../dialogs/response_dialog.dart'; 
import '../../dialogs/yes_no_dialog.dart';    // 예/아니오 모달
import '../../dialogs/room_setting_dialog.dart';
import '../../dialogs/user_info_private_dialog.dart';
import 'playing_private_page.dart';

class WaitingRoomPrivatePage extends StatefulWidget {
  final int roomSeq;
  final String roomType; // "private"

  const WaitingRoomPrivatePage({
    Key? key,
    required this.roomSeq,
    required this.roomType,
  }) : super(key: key);

  @override
  State<WaitingRoomPrivatePage> createState() => _WaitingRoomPrivatePageState();
}

class _WaitingRoomPrivatePageState extends State<WaitingRoomPrivatePage> {
  // ─────────────────────────────────────────
  // 방 설정
  // ─────────────────────────────────────────
  String roomMasterYn = 'N';
  String roomTitle = '';
  String roomIntro = '';
  String openYn = 'Y';
  String roomPw = '';
  int runningTime = 1;
  int numberOfPeople = 10;
  String scoreOpenRange = 'PRIVATE';

  // 유저 목록
  List<Map<String, dynamic>> _userList = [];

  bool _isLoading = true;

  // FRD
  late DatabaseReference _roomRef;
  Stream<DatabaseEvent>? _roomStream;

  // 진행중 화면 이동 중복 방지
  bool _movedToRunningPage = false;

  // 강퇴 안내 중복 방지
  bool _kickedOut = false;

  // FRD 구독 해제
  StreamSubscription<DatabaseEvent>? _roomStreamSubscription;

  // (예) 내 user_seq
  String mySeq = '0'; // 원래 '6' 고정이었던 부분 제거

  @override
  void initState() {
    super.initState();
    _loadMySeq();
  }

  /// (A) my_user_seq 로드 -> 리스너
  Future<void> _loadMySeq() async {
    final prefs = await SharedPreferences.getInstance();
    mySeq = prefs.getInt('my_user_seq')?.toString() ?? '0';

    final roomKey = 'korea-${widget.roomSeq}';
    _roomRef = FirebaseDatabase.instance.ref('rooms/$roomKey');
    _listenRoomData();
  }

  void _listenRoomData() {
    _roomStream = _roomRef.onValue;
    _roomStream?.listen((event) {
      final snapshot = event.snapshot;
      if (!snapshot.exists) {
        setState(() {
          _isLoading = false;
          roomTitle = '방 정보 없음';
          _userList = [];
        });
        return;
      }

      final data = snapshot.value as Map<dynamic, dynamic>? ?? {};
      final roomInfoData = data['roomInfo'] as Map<dynamic, dynamic>? ?? {};
      final userInfoData = data['userInfo'] as Map<dynamic, dynamic>? ?? {};

      final roomStatus = (roomInfoData['room_status'] ?? 'WAIT').toString().toUpperCase();

      setState(() {
        roomTitle      = (roomInfoData['room_title']       ?? '') as String;
        roomIntro      = (roomInfoData['room_intro']       ?? '') as String;
        openYn         = (roomInfoData['open_yn']          ?? 'Y') as String;
        roomPw         = (roomInfoData['room_pw']          ?? '') as String;
        runningTime    = _toInt(roomInfoData['running_time'], 1);
        numberOfPeople = _toInt(roomInfoData['number_of_people'], 10);
        scoreOpenRange = (roomInfoData['score_open_range'] ?? 'PRIVATE') as String;

        // 방장 여부
        roomMasterYn = 'N';
        final masterSeq = roomInfoData['master_user_seq'];
        if (masterSeq != null && masterSeq.toString() == mySeq) {
          roomMasterYn = 'Y';
        }

        // 유저 목록
        final tempList = <Map<String, dynamic>>[];
        userInfoData.forEach((userSeq, userMap) {
          tempList.add({
            'user_seq': userSeq,
            'participant_type': userMap['participant_type'] ?? '',
            'nickname': userMap['nickname'] ?? '유저',
            'score': userMap['score'] ?? 0,
            'profile_img': userMap['profile_img'] ?? '',
            'department': userMap['department'] ?? '',
            'introduce_myself': userMap['introduce_myself'] ?? '',
            'ready_yn': (userMap['ready_yn'] ?? 'N').toString().toUpperCase(),
          });
        });
        _userList = tempList;
        _isLoading = false;
      });

      // 진행중 -> 화면 이동
      if (roomStatus == 'RUNNING' && !_movedToRunningPage) {
        _movedToRunningPage = true;
        Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (_) => PlayingPrivatePage(
              roomSeq: widget.roomSeq,
              roomTitle: roomTitle,
            ),
          ),
        );
        return;
      }

      // (2) 여기서 "내 user_seq가 목록에 있는지" 검사
      final amIStillHere = _userList.any((u) => u['user_seq'].toString() == mySeq);

      // (3) 만약 내가 목록에서 사라졌고, 
      //     아직 안내하지 않았으며(_kickedOut == false),
      //     내가 방장도 아니고(roomMasterYn != 'Y'),
      //     방이 진행중/종료가 아닌 상태면 => "강퇴"로 간주
      if (!amIStillHere && !_kickedOut && roomMasterYn != 'Y') {
        // ※ 방이 이미 RUNNING이면 그냥 게임중에 퇴장인지, 
        //    방이 DELETE 상태인지 등 필요 시 조건 보강

        _kickedOut = true; // 중복 안내 막기

        // (★) 강퇴 안내 + 메인으로 이동
        Future.delayed(Duration.zero, () async {
          await showResponseDialog(context, '안내', '강퇴되었습니다.');
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (_) => const MainPage()),
          );
        });
      }
    }, onError: (error) {
      print('FRD onError: $error');
      setState(() {
        _isLoading = false;
        roomTitle = '오류 발생';
      });
    });
  }

  @override
  void dispose() {
    _roomStreamSubscription?.cancel();  // ← 구독 해제
    super.dispose();
  }

  /// (B) 뒤로가기 -> 방 나가기
  Future<void> _onLeaveRoom() async {
    if (roomMasterYn == 'Y') {
      // 방장
      final confirm = await showDialog<bool>(
        context: context,
        barrierDismissible: false,
        builder: (_) {
          return AlertDialog(
            backgroundColor: Colors.white,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(16),
              side: const BorderSide(color: Colors.black, width: 2),
            ),
            title: const Center(
              child: Text(
                '방 나가기',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold, color: Colors.black),
              ),
            ),
            content: const Text(
              '방장이 나가면 방은 삭제됩니다.\n정말 나가시겠습니까?',
              style: TextStyle(fontSize: 14, color: Colors.black),
            ),
            actionsAlignment: MainAxisAlignment.spaceEvenly,
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context, false),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                  padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 12),
                  shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                ),
                child: const Text('취소'),
              ),
              TextButton(
                onPressed: () => Navigator.pop(context, true),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                  padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 12),
                  shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                ),
                child: const Text('확인'),
              ),
            ],
          );
        },
      );
      if (confirm != true) return;

      // leave API
      try {
        final reqBody = {"room_seq": "${widget.roomSeq}"};
        final response = await Api.serverRequest(uri: '/room/score/game/leave', body: reqBody);
        if (response['result'] == 'OK') {
          final resp = response['response'] ?? {};
          if (resp['result'] == 'OK') {
            Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
          } else {
            final msg = resp['response_info']?['msg_content'] ?? '방 나가기 실패';
            final again = await showYesNoDialog(
              context: context,
              title: '오류',
              message: '$msg\n그래도 나가시겠습니까?',
              yesNo: true,
            );
            if (again == true) {
              Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
            }
          }
        } else {
          final again = await showYesNoDialog(
            context: context,
            title: '오류',
            message: '서버오류\n그래도 나가시겠습니까?',
            yesNo: true,
          );
          if (again == true) {
            Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
          }
        }
      } catch (e) {
        final again = await showYesNoDialog(
          context: context,
          title: '오류',
          message: '$e\n그래도 나가시겠습니까?',
          yesNo: true,
        );
        if (again == true) {
          Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
        }
      }
    } else {
      // 일반
      try {
        final reqBody = {"room_seq": "${widget.roomSeq}"};
        final response = await Api.serverRequest(uri: '/room/score/game/leave', body: reqBody);
        if (response['result'] == 'OK') {
          final resp = response['response'] ?? {};
          if (resp['result'] == 'OK') {
            Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
          } else {
            final msg = resp['response_info']?['msg_content'] ?? '나가기 실패';
            final again = await showYesNoDialog(context: context, title: '오류', message: msg, yesNo: true);
            if (again == true) {
              Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
            }
          }
        } else {
          final again = await showYesNoDialog(context: context, title: '오류', message: '서버 통신 오류', yesNo: true);
          if (again == true) {
            Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
          }
        }
      } catch (e) {
        final again = await showYesNoDialog(context: context, title: '오류', message: '$e', yesNo: true);
        if (again == true) {
          Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
        }
      }
    }
  }

  int _toInt(dynamic val, int defaultVal) {
    if (val == null) return defaultVal;
    if (val is int) return val;
    if (val is String) {
      return int.tryParse(val) ?? defaultVal;
    }
    return defaultVal;
  }

  /// 상단 버튼 (방장=3개, 일반=2개)
  Widget _buildTopButtons() {
    if (_isLoading) return const SizedBox();

    final me = _userList.firstWhere(
      (u) => (u['user_seq'] ?? '0') == mySeq,
      orElse: () => {},
    );
    final myReadyYn = (me['ready_yn'] ?? 'N').toString().toUpperCase();
    final bool isReady = (myReadyYn == 'Y');
    final String readyLabel = isReady ? '준비완료' : '준비';

    final btnStyle = ElevatedButton.styleFrom(
      backgroundColor: Colors.white,
      foregroundColor: Colors.black,
      side: const BorderSide(color: Colors.black, width: 1),
    );

    if (roomMasterYn == 'Y') {
      // 방장 => 3개
      return Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onOpenRoomSetting,
                child: const Text('방 설정'),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onToggleReady,
                child: Text(readyLabel),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onGameStart,
                child: const Text('게임 시작'),
              ),
            ),
          ),
        ],
      );
    } else {
      // 일반 => 2개
      return Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 8),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onOpenRoomSetting,
                child: const Text('방 설정'),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 8),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onToggleReady,
                child: Text(readyLabel),
              ),
            ),
          ),
        ],
      );
    }
  }

  /// READY 토글
  Future<void> _onToggleReady() async {
    try {
      final me = _userList.firstWhere(
        (u) => (u['user_seq'] ?? '0') == mySeq,
        orElse: () => {},
      );
      final myReadyYn = (me['ready_yn'] ?? 'N').toString().toUpperCase();
      final bool isReady = (myReadyYn == 'Y');
      final newYn = isReady ? 'N' : 'Y';

      final userRef = _roomRef.child('userInfo').child(mySeq);
      await userRef.update({"ready_yn": newYn});
    } catch (e) {
      print('READY 설정 실패: $e');
    }
  }

  /// 방 설정
  Future<void> _onOpenRoomSetting() async {
    final roomInfo = {
      "room_seq": "${widget.roomSeq}",
      "room_master_yn": roomMasterYn,
      "room_title": roomTitle,
      "room_intro": roomIntro,
      "open_yn": openYn,
      "room_pw": roomPw,
      "running_time": runningTime,
      "room_type": widget.roomType,
      "number_of_people": numberOfPeople,
      "score_open_range": scoreOpenRange,
    };

    final result = await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (_) => RoomSettingModal(roomInfo: roomInfo),
    );
    if (result == 'refresh') {
      // ...
    }
  }

  /// 게임 시작
  Future<void> _onGameStart() async {
    final notReady = _userList.any((u) {
      final ry = (u['ready_yn'] ?? 'N').toString().toUpperCase();
      return (ry != 'Y');
    });
    if (notReady) {
      showResponseDialog(context, '안내', 'READY되지 않은 참가자가 있습니다(방장 포함).');
      return;
    }

    final requestBody = {
      "room_seq": "${widget.roomSeq}",
      "room_type": "PRIVATE",
    };

    try {
      final response = await Api.serverRequest(
        uri: '/room/score/game/start',
        body: requestBody,
      );
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          print('게임 시작 요청 성공(개인전)');
        } else {
          final msgTitle = resp['response_info']?['msg_title'] ?? '오류';
          final msgContent = resp['response_info']?['msg_content'] ?? '게임 시작 실패';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신 오류');
      }
    } catch (e) {
      showResponseDialog(context, '오류', '$e');
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        backgroundColor: Colors.black,
        elevation: 0,
        title: const Text('대기 방 (개인전)', style: TextStyle(color: Colors.white)),
        leading: IconButton(
          icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
          onPressed: _onLeaveRoom, 
        ),
      ),
      bottomNavigationBar: Container(
        height: 50,
        decoration: BoxDecoration(
          color: Colors.grey.shade300,
          border: Border.all(color: Colors.black, width: 1),
        ),
        child: const Center(
          child: Text('구글 광고', style: TextStyle(color: Colors.black)),
        ),
      ),
      body: _isLoading
          ? const Center(child: CircularProgressIndicator())
          : SingleChildScrollView(
              padding: const EdgeInsets.symmetric(vertical: 20, horizontal: 16),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Center(
                    child: Text(
                      roomTitle.isNotEmpty ? roomTitle : '방 제목',
                      style: const TextStyle(fontSize: 24, fontWeight: FontWeight.bold, color: Colors.black),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  const SizedBox(height: 8),

                  _buildTopButtons(),
                  const SizedBox(height: 20),

                  const Text('사회자', style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold, color: Colors.black)),
                  const SizedBox(height: 8),
                  _buildAdminSection(),
                  const SizedBox(height: 20),

                  const Text('참가자', style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold, color: Colors.black)),
                  const SizedBox(height: 8),
                  _buildPlayerSection(),
                ],
              ),
            ),
    );
  }

  // 사회자
  Widget _buildAdminSection() {
    final adminList = _userList.where((u) {
      final t = (u['participant_type'] ?? '').toString().toUpperCase();
      return t == 'ADMIN';
    }).toList();

    return Container(
      width: double.infinity,
      padding: const EdgeInsets.all(8),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: adminList.isEmpty
          ? const Text('사회자가 없습니다.', style: TextStyle(color: Colors.black))
          : Wrap(
              spacing: 16,
              runSpacing: 8,
              children: adminList.map(_buildSeat).toList(),
            ),
    );
  }

  // 일반 참가자
  Widget _buildPlayerSection() {
    final playerList = _userList.where((u) {
      final t = (u['participant_type'] ?? '').toString().toUpperCase();
      return t != 'ADMIN';
    }).toList();

    return Container(
      width: double.infinity,
      padding: const EdgeInsets.all(8),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: playerList.isEmpty
          ? const Text('참가자가 없습니다.', style: TextStyle(color: Colors.black))
          : SingleChildScrollView(
              child: Wrap(
                spacing: 16,
                runSpacing: 16,
                alignment: WrapAlignment.center,
                children: playerList.map(_buildSeat).toList(),
              ),
            ),
    );
  }

  // Seat
  Widget _buildSeat(Map<String, dynamic> userData) {
    final userName = userData['nickname'] ?? '유저';
    final profileImg = userData['profile_img'] ?? '';
    final readyYn = userData['ready_yn'] ?? 'N';
    final isReady = (readyYn == 'Y');
    final isMaster = (roomMasterYn == 'Y');

    return GestureDetector(
      onTap: () async {
        final result = await showDialog(
          context: context,
          barrierDismissible: false,
          builder: (_) => UserInfoPrivateDialog(
            userData: userData,
            isRoomMaster: isMaster,
            roomSeq: widget.roomSeq,
            roomTypeName: 'PRIVATE',
          ),
        );
        if (result == 'refresh') {
          // ...
        }
      },
      child: Container(
        margin: const EdgeInsets.only(right: 8),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Container(
              width: 50,
              height: 50,
              decoration: BoxDecoration(
                color: Colors.white,
                border: Border.all(
                  color: isReady ? Colors.red : Colors.black,
                  width: isReady ? 2 : 1,
                ),
                borderRadius: BorderRadius.circular(20),
                boxShadow: isReady
                    ? [
                        BoxShadow(
                          color: Colors.redAccent.withOpacity(0.6),
                          blurRadius: 8,
                          spreadRadius: 2,
                          offset: const Offset(0, 0),
                        )
                      ]
                    : [],
              ),
              child: ClipRRect(
                borderRadius: BorderRadius.circular(20),
                child: Image.network(
                  'https://eldsoft.com:8097/images$profileImg',
                  fit: BoxFit.cover,
                  errorBuilder: (ctx, err, st) {
                    return const Center(
                      child: Text(
                        '이미지\n불가',
                        textAlign: TextAlign.center,
                        style: TextStyle(fontSize: 10),
                      ),
                    );
                  },
                ),
              ),
            ),
            const SizedBox(height: 4),
            Text(userName, style: const TextStyle(fontSize: 12, color: Colors.black)),
          ],
        ),
      ),
    );
  }
}
