import 'package:flutter/material.dart';
import 'dart:async';
import 'package:firebase_database/firebase_database.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'main_page.dart';
import '../../plugins/api.dart'; // 서버 API
import '../../dialogs/response_dialog.dart'; // 응답 모달
import '../../dialogs/yes_no_dialog.dart';    // 예/아니오 모달
import '../../dialogs/room_setting_dialog.dart';
import '../../dialogs/user_info_team_dialog.dart';
import '../../dialogs/team_name_edit_dialog.dart';
import 'playing_team_page.dart';

class WaitingRoomTeamPage extends StatefulWidget {
  final int roomSeq;
  final String roomType; // "team"

  const WaitingRoomTeamPage({
    Key? key,
    required this.roomSeq,
    required this.roomType,
  }) : super(key: key);

  @override
  State<WaitingRoomTeamPage> createState() => _WaitingRoomTeamPageState();
}

class _WaitingRoomTeamPageState extends State<WaitingRoomTeamPage> {
  // ─────────────────────────────────────────
  // 방 설정
  // ─────────────────────────────────────────
  String roomMasterYn = 'N';
  String roomTitle = '';
  String roomIntro = '';
  String openYn = 'Y';
  String roomPw = '';
  int runningTime = 1;
  int numberOfPeople = 10;
  String scoreOpenRange = 'PRIVATE';
  int numberOfTeams = 1;

  // 팀명 리스트
  List<String> _teamNameList = [];

  // 유저 목록
  List<Map<String, dynamic>> _userList = [];

  bool _isLoading = true;

  // FRD
  late DatabaseReference _roomRef;
  Stream<DatabaseEvent>? _roomStream;

  // 진행중 화면 중복 이동 방지
  bool _movedToRunningPage = false;

  // 강퇴 안내 중복 방지
  bool _kickedOut = false;

  // FRD 구독 해제
  StreamSubscription<DatabaseEvent>? _roomStreamSubscription;

  // 로컬스토리지에서 가져올 user_seq
  String mySeq = '0'; // 원래 '6'을 하드코딩 했던 부분을 제거

  @override
  void initState() {
    super.initState();
    _loadMySeq();
  }

  /// (A) 내 user_seq를 로드하고 나서 방 레퍼런스 설정 + 리스너 등록
  Future<void> _loadMySeq() async {
    final prefs = await SharedPreferences.getInstance();
    // 예: 저장된 자료형에 따라 getString or getInt
    mySeq = prefs.getInt('my_user_seq')?.toString() ?? '0';
    // roomKey / FRD 설정
    final roomKey = 'korea-${widget.roomSeq}';
    _roomRef = FirebaseDatabase.instance.ref('rooms/$roomKey');
    // 리스너 시작
    _listenRoomData();
  }

  @override
  void dispose() {
    _roomStreamSubscription?.cancel();  // ← 구독 해제
    super.dispose();
  }

  void _listenRoomData() {
    _roomStream = _roomRef.onValue;
    _roomStream?.listen((event) async {
      final snapshot = event.snapshot;
      if (!snapshot.exists) {
        setState(() {
          _isLoading = false;
          roomTitle = '방 정보 없음';
          _userList = [];
        });
        return;
      }

      final data = snapshot.value as Map<dynamic, dynamic>? ?? {};
      final roomInfoData = data['roomInfo'] as Map<dynamic, dynamic>? ?? {};
      final userInfoData = data['userInfo'] as Map<dynamic, dynamic>? ?? {};

      // 현재 방 상태
      final roomStatus = (roomInfoData['room_status'] ?? 'WAIT').toString().toUpperCase();

      setState(() {
        roomTitle      = (roomInfoData['room_title']       ?? '') as String;
        roomIntro      = (roomInfoData['room_intro']       ?? '') as String;
        openYn         = (roomInfoData['open_yn']          ?? 'Y') as String;
        roomPw         = (roomInfoData['room_pw']          ?? '') as String;
        runningTime    = _toInt(roomInfoData['running_time'], 1);
        numberOfPeople = _toInt(roomInfoData['number_of_people'], 10);
        scoreOpenRange = (roomInfoData['score_open_range']  ?? 'PRIVATE') as String;
        numberOfTeams  = _toInt(roomInfoData['number_of_teams'], 1);

        // 팀명 리스트
        final tStr = (roomInfoData['team_name_list'] ?? '') as String;
        if (tStr.isNotEmpty) {
          _teamNameList = tStr.split(',').map((e) => e.trim().toUpperCase()).toList();
        } else {
          _teamNameList = List.generate(numberOfTeams, (i) => String.fromCharCode(65 + i));
        }

        // 방장 여부
        roomMasterYn = 'N';
        final masterSeq = roomInfoData['master_user_seq'];
        if (masterSeq != null && masterSeq.toString() == mySeq) {
          roomMasterYn = 'Y';
        }

        // 유저 목록
        final tempList = <Map<String, dynamic>>[];
        userInfoData.forEach((userSeq, userMap) {
          tempList.add({
            'user_seq': userSeq,
            'participant_type': userMap['participant_type'] ?? '',
            'nickname': userMap['nickname'] ?? '유저',
            'team_name': userMap['team_name'] ?? '',
            'score': userMap['score'] ?? 0,
            'profile_img': userMap['profile_img'] ?? '',
            'department': userMap['department'] ?? '',
            'introduce_myself': userMap['introduce_myself'] ?? '',
            'ready_yn': (userMap['ready_yn'] ?? 'N').toString().toUpperCase(),
          });
        });
        _userList = tempList;
        _isLoading = false;
      });

      // 상태가 RUNNING이면 진행중 화면으로
      if (roomStatus == 'RUNNING' && !_movedToRunningPage) {
        _movedToRunningPage = true;
        Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (_) => PlayingTeamPage(
              roomSeq: widget.roomSeq,
              roomTitle: roomTitle,
            ),
          ),
        );
        return;
      }

      // (2) 여기서 "내 user_seq가 목록에 있는지" 검사
      final amIStillHere = _userList.any((u) => u['user_seq'].toString() == mySeq);

      // (3) 만약 내가 목록에서 사라졌고, 
      //     아직 안내하지 않았으며(_kickedOut == false),
      //     내가 방장도 아니고(roomMasterYn != 'Y'),
      //     방이 진행중/종료가 아닌 상태면 => "강퇴"로 간주
      if (!amIStillHere && !_kickedOut && roomMasterYn != 'Y') {
        // ※ 방이 이미 RUNNING이면 그냥 게임중에 퇴장인지, 
        //    방이 DELETE 상태인지 등 필요 시 조건 보강

        _kickedOut = true; // 중복 안내 막기

        // (★) 강퇴 안내 + 메인으로 이동
        Future.delayed(Duration.zero, () async {
          await showResponseDialog(context, '안내', '강퇴되었습니다.');
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (_) => const MainPage()),
          );
        });
      }
    }, onError: (error) {
      print('FRD onError: $error');
      setState(() {
        _isLoading = false;
        roomTitle = '오류 발생';
      });
    });
  }

  // ─────────────────────────────────────────
  // [추가] 뒤로가기 -> 방 나가기
  // ─────────────────────────────────────────
  Future<void> _onLeaveRoom() async {
    if (roomMasterYn == 'Y') {
      // 방장 -> 경고 모달
      final confirm = await showDialog<bool>(
        context: context,
        barrierDismissible: false,
        builder: (_) {
          return AlertDialog(
            backgroundColor: Colors.white,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(16),
              side: const BorderSide(color: Colors.black, width: 2),
            ),
            title: const Center(
              child: Text(
                '방 나가기',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold, color: Colors.black),
              ),
            ),
            content: const Text(
              '방장이 나가면 방은 삭제됩니다.\n정말 나가시겠습니까?',
              style: TextStyle(fontSize: 14, color: Colors.black),
            ),
            actionsAlignment: MainAxisAlignment.spaceEvenly,
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context, false),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                  padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 12),
                  shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                ),
                child: const Text('취소'),
              ),
              TextButton(
                onPressed: () => Navigator.pop(context, true),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                  padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 12),
                  shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                ),
                child: const Text('확인'),
              ),
            ],
          );
        },
      );
      if (confirm != true) return;

      try {
        final reqBody = {"room_seq": "${widget.roomSeq}"};
        final response = await Api.serverRequest(uri: '/room/score/game/leave', body: reqBody);
        if (response['result'] == 'OK') {
          final resp = response['response'];
          if (resp != null && resp['result'] == 'OK') {
            Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
          } else {
            final msg = resp?['response_info']?['msg_content'] ?? '방 나가기 실패';
            final again = await showYesNoDialog(
              context: context,
              title: '오류',
              message: '$msg\n그래도 나가시겠습니까?',
              yesNo: true,
            );
            if (again == true) {
              Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
            }
          }
        } else {
          final again = await showYesNoDialog(
            context: context,
            title: '오류',
            message: '서버오류\n그래도 나가시겠습니까?',
            yesNo: true,
          );
          if (again == true) {
            Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
          }
        }
      } catch (e) {
        final again = await showYesNoDialog(
          context: context,
          title: '오류',
          message: '$e\n그래도 나가시겠습니까?',
          yesNo: true,
        );
        if (again == true) {
          Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
        }
      }
    } else {
      // 일반 유저
      try {
        final reqBody = {"room_seq": "${widget.roomSeq}"};
        final response = await Api.serverRequest(uri: '/room/score/game/leave', body: reqBody);
        if (response['result'] == 'OK') {
          final resp = response['response'] ?? {};
          if (resp['result'] == 'OK') {
            Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
          } else {
            final msg = resp['response_info']?['msg_content'] ?? '나가기 실패';
            final again = await showYesNoDialog(context: context, title: '오류', message: msg, yesNo: true);
            if (again == true) {
              Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
            }
          }
        } else {
          final again = await showYesNoDialog(context: context, title: '오류', message: '서버 통신 오류', yesNo: true);
          if (again == true) {
            Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
          }
        }
      } catch (e) {
        final again = await showYesNoDialog(context: context, title: '오류', message: '$e', yesNo: true);
        if (again == true) {
          Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
        }
      }
    }
  }

  int _toInt(dynamic val, int defaultVal) {
    if (val == null) return defaultVal;
    if (val is int) return val;
    if (val is String) {
      return int.tryParse(val) ?? defaultVal;
    }
    return defaultVal;
  }

  // ─────────────────────────────────────────
  // 상단 버튼들: 방장 = 3개, 일반 = 2개
  // READY 버튼에 "준비"/"준비완료" 표시
  // 게임 시작: 전체 READY=Y 필요
  // ─────────────────────────────────────────
  Widget _buildTopButtons() {
    if (_isLoading) return const SizedBox();

    // 내 READY 상태
    final me = _userList.firstWhere(
      (u) => (u['user_seq'] ?? '0') == mySeq,
      orElse: () => {},
    );
    final myReadyYn = (me['ready_yn'] ?? 'N').toString().toUpperCase();
    final bool isReady = (myReadyYn == 'Y');
    final String readyLabel = isReady ? '준비완료' : '준비';

    final btnStyle = ElevatedButton.styleFrom(
      backgroundColor: Colors.white,
      foregroundColor: Colors.black,
      side: const BorderSide(color: Colors.black, width: 1),
    );

    if (roomMasterYn == 'Y') {
      // 방장 -> [방 설정], [준비/준비완료], [게임 시작]
      return Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onOpenRoomSetting,
                child: const Text('방 설정'),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onToggleReady,
                child: Text(readyLabel),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onGameStart,
                child: const Text('게임 시작'),
              ),
            ),
          ),
        ],
      );
    } else {
      // 일반 -> [방 설정], [준비/준비완료]
      return Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 8),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onOpenRoomSetting,
                child: const Text('방 설정'),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 8),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onToggleReady,
                child: Text(readyLabel),
              ),
            ),
          ),
        ],
      );
    }
  }

  /// READY 토글
  Future<void> _onToggleReady() async {
    try {
      // 내 데이터
      final me = _userList.firstWhere(
        (u) => (u['user_seq'] ?? '') == mySeq,
        orElse: () => {},
      );
      final myReadyYn = (me['ready_yn'] ?? 'N').toString().toUpperCase();
      final bool isReady = (myReadyYn == 'Y');
      final newYn = isReady ? 'N' : 'Y';

      final userRef = _roomRef.child('userInfo').child(mySeq);
      await userRef.update({"ready_yn": newYn});
    } catch (e) {
      print('READY 설정 실패: $e');
    }
  }

  /// 방 설정 열기
  Future<void> _onOpenRoomSetting() async {
    final roomInfo = {
      "room_seq": "${widget.roomSeq}",
      "room_master_yn": roomMasterYn,
      "room_title": roomTitle,
      "room_intro": roomIntro,
      "open_yn": openYn,
      "room_pw": roomPw,
      "running_time": runningTime,
      "room_type": widget.roomType,
      "number_of_people": numberOfPeople,
      "score_open_range": scoreOpenRange,
      "number_of_teams": numberOfTeams,
      "team_name_list": _teamNameList.join(','),
    };

    final result = await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (_) => RoomSettingModal(roomInfo: roomInfo),
    );
    if (result == 'refresh') {
      // do something
    }
  }

  /// 게임 시작 (전체 READY=Y 필요)
  Future<void> _onGameStart() async {
    final notReady = _userList.any((u) {
      final ry = (u['ready_yn'] ?? 'N').toString().toUpperCase();
      return (ry != 'Y');
    });
    if (notReady) {
      showResponseDialog(context, '안내', 'READY되지 않은 참가자가 있습니다(방장 포함).');
      return;
    }

    final requestBody = {
      "room_seq": "${widget.roomSeq}",
      "room_type": "TEAM",
    };
    try {
      final response = await Api.serverRequest(
        uri: '/room/score/game/start',
        body: requestBody,
      );
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          print('게임 시작 요청 성공(팀전)');
        } else {
          final msgTitle = resp['response_info']?['msg_title'] ?? '오류';
          final msgContent = resp['response_info']?['msg_content'] ?? '게임 시작 실패';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신 오류');
      }
    } catch (e) {
      showResponseDialog(context, '오류', '$e');
    }
  }

  // ─────────────────────────────────────────
  // 사회자 / 팀 섹션 / 대기중 / Seat
  // ─────────────────────────────────────────
  Widget _buildAdminSection() {
    final adminList = _userList.where((u) {
      final pType = (u['participant_type'] ?? '').toString().toUpperCase();
      return pType == 'ADMIN';
    }).toList();

    return Container(
      width: double.infinity,
      padding: const EdgeInsets.all(8),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: adminList.isEmpty
          ? const Text('사회자가 없습니다.', style: TextStyle(color: Colors.black))
          : Wrap(
              spacing: 16,
              runSpacing: 8,
              children: adminList.map(_buildSeat).toList(),
            ),
    );
  }

  Widget _buildTeamSection() {
    final players = _userList.where((u) {
      final pType = (u['participant_type'] ?? '').toString().toUpperCase();
      return (pType != 'ADMIN');
    }).toList();

    final Map<String, List<Map<String, dynamic>>> teamMap = {};
    for (final tName in _teamNameList) {
      teamMap[tName] = [];
    }

    for (var user in players) {
      final tName = (user['team_name'] ?? '').toString().trim().toUpperCase();
      if (tName.isNotEmpty && tName != 'WAIT' && teamMap.containsKey(tName)) {
        teamMap[tName]!.add(user);
      }
    }

    if (teamMap.isEmpty) {
      return Container(
        width: double.infinity,
        padding: const EdgeInsets.all(8),
        decoration: BoxDecoration(
          color: Colors.white,
          border: Border.all(color: Colors.black),
          borderRadius: BorderRadius.circular(8),
        ),
        child: const Text('팀에 배정된 참가자가 없습니다.', style: TextStyle(color: Colors.black)),
      );
    }

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: _teamNameList.map((teamName) {
        final members = teamMap[teamName]!;

        return Container(
          margin: const EdgeInsets.only(bottom: 16),
          decoration: BoxDecoration(
            color: Colors.white,
            border: Border.all(color: Colors.black),
            borderRadius: BorderRadius.circular(8),
          ),
          child: Column(
            children: [
              GestureDetector(
                onTap: () async {
                  if (roomMasterYn == 'Y') {
                    final result = await showDialog(
                      context: context,
                      barrierDismissible: false,
                      builder: (_) => TeamNameEditModal(
                        roomSeq: widget.roomSeq,
                        roomTypeName: 'TEAM',
                        beforeTeamName: teamName,
                        existingTeamNames: _teamNameList,
                      ),
                    );
                    if (result == 'refresh') {
                      // ...
                    }
                  }
                },
                child: Container(
                  width: double.infinity,
                  padding: const EdgeInsets.symmetric(vertical: 8),
                  color: Colors.black,
                  child: Center(
                    child: Text(
                      '팀 $teamName',
                      style: const TextStyle(color: Colors.white, fontWeight: FontWeight.bold),
                    ),
                  ),
                ),
              ),
              SingleChildScrollView(
                scrollDirection: Axis.horizontal,
                padding: const EdgeInsets.all(8),
                child: Row(children: members.map(_buildSeat).toList()),
              ),
            ],
          ),
        );
      }).toList(),
    );
  }

  Widget _buildWaitSection() {
    final waitList = _userList.where((u) {
      final pType = (u['participant_type'] ?? '').toString().toUpperCase();
      if (pType == 'ADMIN') return false;
      final tName = (u['team_name'] ?? '').toString().toUpperCase();
      return (tName.isEmpty || tName == 'WAIT');
    }).toList();

    if (waitList.isEmpty) return const SizedBox();

    return Container(
      margin: const EdgeInsets.only(bottom: 16),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: Column(
        children: [
          Container(
            width: double.infinity,
            padding: const EdgeInsets.symmetric(vertical: 8),
            color: Colors.black,
            child: const Center(
              child: Text('대기중', style: TextStyle(color: Colors.white, fontWeight: FontWeight.bold)),
            ),
          ),
          SingleChildScrollView(
            scrollDirection: Axis.horizontal,
            padding: const EdgeInsets.all(8),
            child: Row(children: waitList.map(_buildSeat).toList()),
          ),
        ],
      ),
    );
  }

  Widget _buildSeat(Map<String, dynamic> user) {
    final userName = user['nickname'] ?? '유저';
    final profileImg = user['profile_img'] ?? '';
    final readyYn = user['ready_yn'] ?? 'N';
    final isReady = (readyYn == 'Y');
    final isMaster = (roomMasterYn == 'Y');

    return GestureDetector(
      onTap: () async {
        // 유저 정보 모달
        final result = await showDialog(
          context: context,
          barrierDismissible: false,
          builder: (_) => UserInfoTeamDialog(
            userData: user,
            isRoomMaster: isMaster,
            roomSeq: widget.roomSeq,
            roomTypeName: widget.roomType.toUpperCase(), // "TEAM"
            teamNameList: _teamNameList,
          ),
        );
        if (result == 'refresh') {
          // ...
        }
      },
      child: Container(
        width: 60,
        margin: const EdgeInsets.only(right: 8),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Container(
              width: 50,
              height: 50,
              decoration: BoxDecoration(
                color: Colors.white,
                border: Border.all(
                  color: isReady ? Colors.red : Colors.black,
                  width: isReady ? 2 : 1,
                ),
                borderRadius: BorderRadius.circular(20),
                boxShadow: isReady
                    ? [
                        BoxShadow(
                          color: Colors.redAccent.withOpacity(0.6),
                          blurRadius: 8,
                          spreadRadius: 2,
                          offset: const Offset(0, 0),
                        )
                      ]
                    : [],
              ),
              child: ClipRRect(
                borderRadius: BorderRadius.circular(20),
                child: Image.network(
                  'https://eldsoft.com:8097/images$profileImg',
                  fit: BoxFit.cover,
                  errorBuilder: (ctx, err, st) {
                    return const Center(
                      child: Text(
                        '이미지\n불가',
                        textAlign: TextAlign.center,
                        style: TextStyle(fontSize: 10),
                      ),
                    );
                  },
                ),
              ),
            ),
            const SizedBox(height: 2),
            Text(userName, style: const TextStyle(fontSize: 12, color: Colors.black)),
          ],
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        backgroundColor: Colors.black,
        elevation: 0,
        title: const Text('대기 방 (팀전)', style: TextStyle(color: Colors.white)),
        leading: IconButton(
          icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
          onPressed: _onLeaveRoom, // 나가기
        ),
      ),
      bottomNavigationBar: Container(
        height: 50,
        decoration: BoxDecoration(
          color: Colors.grey.shade300,
          border: Border.all(color: Colors.black, width: 1),
        ),
        child: const Center(
          child: Text('구글 광고', style: TextStyle(color: Colors.black)),
        ),
      ),
      body: _isLoading
          ? const Center(child: CircularProgressIndicator())
          : SingleChildScrollView(
              padding: const EdgeInsets.symmetric(vertical: 16, horizontal: 16),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Center(
                    child: Text(
                      roomTitle.isNotEmpty ? roomTitle : '방 제목',
                      style: const TextStyle(fontSize: 24, fontWeight: FontWeight.bold, color: Colors.black),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  const SizedBox(height: 8),

                  _buildTopButtons(),
                  const SizedBox(height: 20),

                  const Text('사회자', style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold, color: Colors.black)),
                  const SizedBox(height: 8),
                  _buildAdminSection(),
                  const SizedBox(height: 20),

                  const Text('팀별 참가자', style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold, color: Colors.black)),
                  const SizedBox(height: 8),
                  _buildTeamSection(),
                  const SizedBox(height: 20),

                  _buildWaitSection(),
                ],
              ),
            ),
    );
  }
}
